<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<!-- forms\admin_framework\admin_server_status_summary.jsp -->
<%@ taglib prefix="c" uri="http://java.sun.com/jsp/jstl/core"%>
<%@ taglib prefix="change" uri="http://www.telelogic.com/change"%>
<%@ taglib prefix="fn" uri = "http://java.sun.com/jsp/jstl/functions"%>
<%--
	admin_server_status_summary.jsp page displays the information about the
	Server configuration. It also displays the Server errors and the Current System Load 
	information on the IBM Rational Change server, which are seperate jsps, through jsp include.
--%>

<!-- /* -->
<!-- * IBM and/or HCL Confidential
<!-- * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
<!-- * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
<!-- *
<!-- *  The source code for this program is not published or otherwise
<!-- *  divested of its trade secrets, irrespective of what has been
<!-- *  deposited with the U.S. Copyright Office.
<!-- */ -->

<html>
	<head>
		<title>
			<change:resource key='productName'></change:resource>
		</title>
	
		<style>
			<change:wsletExpander>
				<!-- WSLET CODE=PTInclude -->
				<!-- PARAM name=styles/default.css -->
				<!-- PARAM name=styles/default_admin.css -->
				<!-- PARAM name=styles/server_status.css -->
				<!-- /WSLET -->
			</change:wsletExpander>
		</style>
		
		<script type="text/javascript" src="<change:urlCreator webResource='JS'/>dwr/engine.js"></script>
		<script type="text/javascript" src="<change:urlCreator webResource='JS'/>dwr/util.js"></script>
		<script type="text/javascript" src="<change:urlCreator webResource='BASEURL'/>/dwr/interface/HomePageUtilsAjax.js"></script>
		<script type="text/javascript" src="<change:urlCreator webResource='JS'/>dojo/dojo/dojo.js"></script>
		
		<script>
			<change:wsletExpander>
				<!-- WSLET CODE=PTInclude -->
					<!-- PARAM name=toggleSection.js -->
					<!-- PARAM name=StyleUtils.js -->
					<!-- PARAM name=domUtilities.js -->
					<!-- PARAM name=SelectUtils.js -->
					<!-- PARAM name=GenericTableSorter.js -->
					<!-- PARAM name=Zebra.js -->
					<!-- PARAM name=ScrapeText.js -->
					<!-- PARAM name=EventUtils.js -->
					<!-- PARAM name=admin_framework/currentSysLoad.js -->
					<!-- PARAM name=admin_framework/utils.js -->
					<!-- PARAM NAME=validation.js -->
				<!-- /WSLET -->
			</change:wsletExpander>
		
			/**
			 * Functions to read the *visible* values of each logical column.
			 * Each array index is a function to read the column with the same index.
			 * Used both in sorting and filtering, which both act on what the user sees
			 * (not necessarily the underlying values of the controls.).
			 */
			var gColumnValueFunctions = [
				function(row)
				{
					return ScrapeText.fromTextNodes(row.cells[0]);
				},
		
				function(row)
				{
					return ScrapeText.fromTextNodes(row.cells[1]);
				},
		
				function(row)
				{
					return ScrapeText.fromTextNodes(row.cells[2]);
				},
		
				function(row)
				{
					return ScrapeText.fromTextNodes(row.cells[3]);
				},
		
				function(row)
				{
					return ScrapeText.fromTextNodes(row.cells[4]);
				},
				
				function(row)
				{
					return ScrapeText.fromTextNodes(row.cells[5]);
				}
			];
		
			var infoTables;
			var dbInfoArray;
				
			/**
			 * Creates table sorters
			 */
			function initOnLoad()
			{
				 <c:if test="${fn:length(statusSummaryForm.map.hostErrorList) > 0}">
					var errHost = new GenericTableSorter(document.getElementById('body-errHost'), gColumnValueFunctions);
					defaultSort(errHost, document.getElementById('sort-on-errHost'), 0);
				 </c:if>
				 
				 <c:if test="${fn:length(statusSummaryForm.map.databaseErrorList) > 0}">
					var errDatabase = new GenericTableSorter(document.getElementById('body-errDatabase'), gColumnValueFunctions);
					defaultSort(errDatabase, document.getElementById('sort-on-errDatabase'), 0);
				 </c:if>
				   
				 <c:if test="${fn:length(statusSummaryForm.map.remoteServerErrorList) > 0}">
					var errRemoteServer = new GenericTableSorter(document.getElementById('body-errRemoteServer'), gColumnValueFunctions);
					defaultSort(errRemoteServer, document.getElementById('sort-on-errRemoteServer'), 0);
				 </c:if>
				 
				 <c:if test="${fn:length(statusSummaryForm.map.licenseErrorList) > 0}">
					var errLicense = new GenericTableSorter(document.getElementById('body-errLicense'), gColumnValueFunctions);
					defaultSort(errLicense, document.getElementById('sort-on-errLicense'), 0);
				 </c:if>
				 
				 populateInfoTable();
				 
				 // Default sort - Sort on database column
				sortCurrentSysLoad('body-curSysLoad', 'sysLoad-td1');
				 
				  //to hide the wait indicator
				 <c:if test="${isCentralServer}">
					document.getElementById("indicator").style.display = "none";
				 </c:if>
			}
			
			function showDetails()
			{
				var selectedServerIndex = document.getElementById("serverList").options.selectedIndex;
				
				var selectedServerName = document.getElementById("serverList").options[selectedServerIndex].text;
				
				if (isCentralServer(document.getElementById("serverList").options[selectedServerIndex].value))
				{
					var centralLabel = trim('<change:resource key="centralServerText"><change:param value=""/></change:resource>');
					var index = selectedServerName.indexOf(centralLabel);

					if (index != 0)
					{
						selectedServerName = trim(selectedServerName.substring(0, index));
					}
				}
				
				document.getElementById("indicator").style.display = "";
				document.getElementById("server-details").style.display = "none";
				
				// Ajax call using dojo. 				
				dojo.xhrGet(
				{
					url: "adminStatusSummaryLoader.do?methodToCall=getServerSystemInfo",					
					content:
					{
						formName: "currentSystemLoad",
						serverName: selectedServerName,
						token: "${user.token}"
					},					
					load: loadServerDetails,
					error: customErrorHandler
				});
			}
			
			function isCentralServer(isCentral)
			{
				if(isCentral == "true")
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			
			function loadServerDetails(serverdetails)
			{
				document.getElementById("indicator").style.display = "none";
				
				processData("server-details", serverdetails);
				populateInfoTable();
						 
				// Default sort - Sort on database column
				if (document.getElementById('sysLoad-td1'))
				{
					sortCurrentSysLoad('body-curSysLoad', 'sysLoad-td1');
				}	
				
				document.getElementById("server-details").style.display = "";
			}
			
			function processData(elementId, data)
			{
				if (data != null || data != '')
				{
					dwr.util.setValue(elementId, data,
					{
						escapeHtml: false
					});
				}
			}
			
			//Error handler for ajax call (if an exception occurs during ajax call)			
			function customErrorHandler(error, args)
			{
				document.getElementById("indicator").style.display = "none";				
				var errMessage = unescape(args.xhr.statusText);
				alert(errMessage.replace(/\+/g, " "));									
				document.getElementById("server-details").style.display = "";						
			}

			function openHelpServer()
			{
				var locale = "lang=<change:wsletExpander><!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=LOCALE --><!-- /WSLET --></change:wsletExpander>";
				document.open(document.getElementById("help_server_url").innerHTML+"?"+locale,'',
				'<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=USER_WINDOW_OPTIONS --><!-- /WSLET -->');
			}
			
			function populateInfoTable()
			{
				var tbodyArr = document.getElementsByTagName("tbody");
				var infoTableIndex = 1;
				
				infoTables = new Array();
				infoTables[0] = new GenericTableSorter(null, gColumnValueFunctions);
				infoTables[0].body = document.getElementById('body-curSysLoad');
				
				for (var tbodyCount = 0; tbodyCount < tbodyArr.length; tbodyCount++)
				{
					if ((tbodyArr[tbodyCount].id != null) || (tbodyArr[tbodyCount].id != undefined))
					{
						bodyId = tbodyArr[tbodyCount].id;
						if (bodyId.indexOf("body-user-") != -1)
						{
							infoTables[infoTableIndex] = new GenericTableSorter(document.getElementById(bodyId), gColumnValueFunctions);
							infoTableIndex++;
						}
						
						if (bodyId.indexOf("body-sessions-") != -1)
						{
							infoTables[infoTableIndex] = new GenericTableSorter(document.getElementById(bodyId), gColumnValueFunctions);
							infoTableIndex++;
						}
					} 
				}
				
				if (document.getElementById('body-total-users'))
				{
					infoTables[infoTableIndex] = new GenericTableSorter(document.getElementById('body-total-users'), gColumnValueFunctions);
					infoTableIndex++;
				}
				
				if (document.getElementById('body-total-sessions'))
				{
					infoTables[infoTableIndex] = new GenericTableSorter(document.getElementById('body-total-sessions'), gColumnValueFunctions);
					infoTableIndex++;
				} 
			}

			EventUtils.addEvent(window, 'load', initOnLoad, false);
		</script>
	</head>
	
	<body id="mainBody">
	
		<table width="100%" class="barHeading">
			<tr>
				<th><change:resource key="statusSummary"/></th>
			</tr>
		</table>
	
		<table class="sys-section" id="conf" width="100%">
			<thead onclick="toggleSection(this);">
				<tr>
					<th><change:resource key="configuration"/></th>
					<th width="80%">&nbsp;</th>
					<th style="text-align: right;">
						<img alt="<change:resource key='altTextDown'/>" src="<change:urlCreator webResource='PTIMAGES'/>down_chevron.gif"
							border="0" style="margin: 0px 7px 0px 1px;">
					</th>
				</tr>
			</thead>
	
			<tbody id="header" style="display: none;">
				<tr>
					<td style="text-align: right; padding-top: 10px; padding-left: 25px;">
						<span class="label_decoration" title="<change:resource key="syneryChangeHomeToolTip"/>">
							<change:resource key="syneryChangeHome"/>
						</span>
					</td>
					<td style="padding-top: 10px;">
						<change:serverProperty property="CHANGE_HOME" />
					</td>
				</tr>
				<tr>
					<td style="text-align: right; padding-bottom: 10px;">
						<span class="label_decoration" title="<change:resource key="synergyCMHomeToolTip"/>">
					  		<change:resource key="synergyCMHome"/>
					  	</span>
					</td>
					<td style="padding-bottom: 10px;">
						<change:serverProperty property="SYNERGY_HOME" />
					</td>
				</tr>
				<tr>
				<c:if test="${isLdapEnabled}">
				<td style="text-align: right; padding-top: 10px; padding-bottom: 10px;">
						<span class="label_decoration" title="<change:resource key="ldapUrlToolTip" bundle="531_patch"/>">
							<change:resource key="ldapURL" bundle="531_patch"/>
						</span>
					</td>
					<td style="padding-top: 10px; padding-bottom: 10px;">
						<change:serverProperty property="TDS_URL" />
					</td>
				</c:if>
				<c:if test="${!isLdapEnabled}">
					<td style="text-align: right; padding-top: 10px; padding-bottom: 10px;">
						<span class="label_decoration" title="<change:resource key="tdsUrlToolTip"/>">
							<change:resource key="tdsUrl"/>
						</span>
					</td>
					<td style="padding-top: 10px; padding-bottom: 10px;">
						<change:serverProperty property="TDS_URL" />
					</td>
				</c:if>
				</tr>
				<c:if test="${!isRemoteServer}">
					<tr>
						<td style="text-align: right; padding-top: 10px;">
							<span class="label_decoration" title="<change:resource key="installedProcessToolTip"/>">
								<change:resource key="installedProcess"/>
							</span>
						</td>
						<td style="padding-top: 10px;">
							<change:serverProperty property="INSTALLED_PROCESS" />
						</td>
					</tr>
					
					<tr>
						<td style="text-align: right; padding-bottom: 10px;">
							<span class="label_decoration" title="<change:resource key="processPackageToolTip"/>">
								<change:resource key="processPackage"/>
							</span>
						</td>
						<td style="padding-bottom: 10px;">
							<change:serverProperty property="PROCESS_PACKAGE" />
						</td>
					</tr>
				</c:if>
				<tr>
					<td style="text-align: right; padding-top: 10px; padding-bottom: 10px">
						<span class="label_decoration" title="<change:resource key="timeZoneToolTip"/>">
							<change:resource key="timeZone" />
						</span>
					</td>
					<td style="padding-top: 10px; padding-bottom: 10px">
						<change:serverProperty property="TIME_ZONE" />
					</td>
				</tr>
				
				<tr>
					<td style="text-align: right; padding-top: 10px; padding-bottom: 10px">
						<span class="label_decoration" title="<change:resource key="helpServerToolTip"/>">
							<change:resource key="helpServerStatusLink" />
						</span>
					</td>
					<td style="padding-top: 10px; padding-bottom: 10px">
						<c:choose> 
  							<c:when test="${isHelpServerConfigured}">
								<U><a HREF="javascript: openHelpServer();" id="help_server_url"><change:serverProperty property="HELP_SERVER_URL"/></a></U>
							</c:when>
							<c:otherwise>
								<change:resource key="helpServerNotConfigured"/>
							</c:otherwise>
						</c:choose>
					</td>
				</tr>

				<c:if test="${!isStandAloneServer}">
					<tr>
						<td style="text-align: right; padding-top: 10px; padding-bottom: 10px;" valign="top">
							<c:if test="${isCentralServer}">
								<span class="label_decoration" title="<change:resource key="remoteServersToolTip"/>">
									<change:resource key="remoteServersLbl"/>
								</span>
							</c:if>
							<c:if test="${isRemoteServer}">
								<span class="label_decoration" title="<change:resource key="centralServerToolTip"/>">
									<change:resource key="centralServerLbl"/>
								</span>
							</c:if>
						</td>
						<td style="padding-top: 10px; padding-bottom: 10px;" valign="top">
							<change:serverProperty property="SERVER_INFO" />
						</td>
					</tr>
				</c:if>
			</tbody>
		</table>
		<br>
		
		<%-- Include Errors Section jsp file--%>	
		<%@ include file="admin_errors_section.jsp" %>
		
		<br>
	   	
	   	<%-- Include Current System Load Section jsp file--%>	
		<%@ include file="admin_current_system_load_section.jsp" %> 
		
	</body>
</html>
