/**
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */

function DraUtils()
{
}

DraUtils.LABEL = 'label_';
DraUtils.LABEL_REQUIRED = DraUtils.LABEL + 'required';
DraUtils.LABEL_DECORATION = DraUtils.LABEL + 'decoration';
DraUtils.REQUIRED_CSS = 'label_required_dra';

DraUtils.setupDra = function()
{
	window.addEventListener("keydown",stopEvent);
	if (IS_DRA_ENABLED)
	{
		var oldHandler;
		var forAttribute;
		var ele;
	
		for (var i = 0; i < DRA_DATA.length ; i++)
		{
			var rule = DRA_DATA[i];
			forAttribute = rule.attribute_name;
			ele = dojo.byId(forAttribute);
			
			if (ele)
			{
				if(!DraUtils._isReadOnlyAttribute(ele))
				{
					if (CtrlUtils.webType(forAttribute) == 'CCM_LISTBOX')
					{
						oldHandler = ele.onchange;
						ele.onchange = DraUtils.makeOnChangeHandler(forAttribute, oldHandler);
					}
					else if (CtrlUtils.webType(forAttribute) == 'CCM_TOGGLE')
					{
						oldHandler = ele.onclick;
						ele.onclick = DraUtils.makeOnClickHandler(forAttribute, oldHandler);
					}
				}
			}
		}
	}
}

function stopEvent(event){
	if(event.code=="Enter"){
		event.preventDefault();
	}
}

DraUtils.makeOnClickHandler = function(attr, oldFunc)
{
	return function()
	{
		DraUtils.callOnClick(attr, oldFunc);
	};
}

DraUtils.callOnClick = function(ruleAttribute, origFunc)
{
	if (origFunc)
	{
		origFunc.call(dojo.byId(ruleAttribute));
	}
	
	DraUtils._enforceDraRuleIfPresent(ruleAttribute);
}


DraUtils.makeOnChangeHandler = function(attr, oldFunc)
{
	return function()
	{
		DraUtils.callOnChange(attr, oldFunc);
	};
}


DraUtils.callOnChange = function(ruleAttribute, origFunc)
{
	if (origFunc)
	{
		origFunc.call()
	}
	
	DraUtils._enforceDraRuleIfPresent(ruleAttribute);
}

DraUtils.applyDraRulesOnLoad = function()
{
	if (IS_DRA_ENABLED)
	{
		var oldHandler;
		var forAttribute;
		var ele;
	
		for (var i = 0; i < DRA_DATA.length ; i++)
		{
			var attrRule = DRA_DATA[i];
			forAttribute = attrRule.attribute_name;
			ele = dojo.byId(forAttribute);
			
			if (ele)
			{
				var valueRules = attrRule.rule;
				var currAttrValue = CtrlUtils.getValue(forAttribute);
				for (var u = 0; u < valueRules.length; u++)
				{
					if (currAttrValue == valueRules[u].value)
					{
						DraUtils._enforceDraRuleIfPresent(forAttribute);
					}
				}
			}
		}
	}
}

DraUtils._enforceDraRuleIfPresent = function(attr)
{
	var webType = CtrlUtils.webType(attr);
	var elementValue = '';
	var attrEle = dojo.byId(attr);
	
	if (DraUtils._isReadOnlyAttribute(attrEle))
	{
		elementValue = CtrlUtils.getValue(attr);
	}
	else
	{
		if (webType == 'CCM_LISTBOX')
		{
			elementValue = SelectUtils.getSelectedText(attr);
		}
		else if (webType == 'CCM_TOGGLE')
		{
			elementValue = (attrEle.checked) ? 'true' : 'false';
		}
	}
	
	DraUtils._revokeDraRulesForOtherValues(attr, elementValue);
	DraUtils._applyDraRuleForAttribute(attr, elementValue);
}


DraUtils._applyDraRuleForAttribute = function(attribute, value)
{
	var DraRule = DraUtils._getDraRule(attribute);
	
	if (DraRule)
	{
		if (DraUtils._isForBulkModification())
		{
			for (var a = 0; a < DraRule.length; a++)
			{
				DraUtils._applyDrarulesPerAttribute(DraRule[a], attribute, value);
			}
		}
		else
		{
			if (DraRule.lifecycle != LIFECYCLE_IN_ACTION)
			{
				return;
			}
			
			DraUtils._applyDrarulesPerAttribute(DraRule, attribute, value);
		}
		
		
	}
}

DraUtils._applyDrarulesPerAttribute = function (DraRule, attribute, value)
{
	var rules = DraRule.rule;
	var reqAttrs = [];
	
	for (var l = 0; l < rules.length; l++ )
	{
		if (value == rules[l].value)
		{
			reqAttrs = rules[l].req_attributes;
		}
	}
	
	for (var r = 0; r < reqAttrs.length; r++)
	{
		var attr = reqAttrs[r];
		var attributeAsElement = dojo.byId(attr);

		if (attributeAsElement != null)
		{
			var attributeLabelAsElement = dojo.byId(DraUtils.LABEL + attr);
			var eleClassName = attributeLabelAsElement.className;
			var eleInnerText = attributeLabelAsElement.innerHTML;
	
			DraUtils._addToMarkedAttributeList(attributeAsElement, attribute);
			attributeLabelAsElement.className = DraUtils.REQUIRED_CSS;
			attributeLabelAsElement.innerHTML = DraUtils._decorateElementLabel(eleInnerText, true);
		}
	}
}

DraUtils._revokeDraRulesForOtherValues = function(attribute, currValue)
{
	var DraRule = DraUtils._getDraRule(attribute);
	if (DraRule)
	{
		if (DraUtils._isForBulkModification())
		{
			for (var a = 0; a < DraRule.length; a++)
			{
				DraUtils._revokeDraRulesForOtherValuesPerAttribute(DraRule[a], attribute, currValue);
			}
		}
		else
		{
			DraUtils._revokeDraRulesForOtherValuesPerAttribute(DraRule, attribute, currValue);
		}
	}
}

DraUtils._revokeDraRulesForOtherValuesPerAttribute = function(DraRule, attribute, currValue)
{
	if (DraRule)
	{
		var allRules = DraRule.rule;
		for (var k = 0; k < allRules.length ; k++ )
		{
			if (allRules[k].value != currValue)
			{
				var reqAttrs = allRules[k].req_attributes;

				for (var r = 0; r < reqAttrs.length; r++)
				{
					var attr = reqAttrs[r];
					var attributeAsElement = dojo.byId(attr);
					
					if (attributeAsElement != null)
					{
						var attributeLabelAsElement = dojo.byId(DraUtils.LABEL + attr);
						var eleClassName = attributeLabelAsElement.className;
						var eleInnerText = attributeLabelAsElement.innerHTML;
	
						if (eleClassName == DraUtils.REQUIRED_CSS)
						{
							var markedByAttrs = DraUtils._removeFormMarkedAttributeList(attributeAsElement, attribute);
							if (markedByAttrs.length <= 0)
							{
								attributeLabelAsElement.className = DraUtils.LABEL_DECORATION;
								attributeLabelAsElement.innerHTML = DraUtils._decorateElementLabel(eleInnerText, false);
							}
						}
					}
				}
			}
		}
	}
}

DraUtils._getDraRule = function(attribute)
{
	
	if (DraUtils._isForBulkModification())
	{
		return DraUtils._getAllDraRulesForAttribute(attribute)
	}
	else
	{
		for (var i = 0; i < DRA_DATA.length ; i++)
		{
			var rule = DRA_DATA[i];
			
			if (attribute == rule.attribute_name && rule.lifecycle == LIFECYCLE_IN_ACTION)
			{
				return rule;
			}
		}
	}
}

DraUtils._getAllDraRulesForAttribute = function (attribute)
{
	var rules = new Array();
	
	for (var i = 0; i < DRA_DATA.length ; i++)
	{
		var rule = DRA_DATA[i];
		
		if (attribute == rule.attribute_name)
		{
			rules.push(rule);
		}
	}
	
	return rules;
}

DraUtils._decorateElementLabel = function(labelText, isRequiredNow)
{
	if(isRequiredNow)
	{
		if (labelText.indexOf('* ') == -1)
		{
			labelText = '* ' + labelText;
		}
	}
	else
	{
		labelText = labelText.substring(2);
	}
	
	return labelText;
}

DraUtils.checkIfDraRulesApplied = function()
{
	var attrNamesNeedvalue = '';
	var attributeNames = '';
	if (IS_DRA_ENABLED)
	{
		var draClassName = '.' + DraUtils.REQUIRED_CSS;
		dojo.query(draClassName).forEach(function(node, index, arr)
		{
		   var labelFor = node.getAttribute('id');
		   labelFor = labelFor.substring(DraUtils.LABEL.length);

		   if (DraUtils._hasEmptyValue(labelFor))
		   {
			   //attribute.isReadOnly = CtrlUtils.isReadOnly(labelFor);
			   
			   if (!CtrlUtils.isReadOnly(labelFor))
			   {
				   var alias = DraUtils._getAttributeAlias(labelFor);
				   labelFor = (alias) ? alias : labelFor;
				   attrNamesNeedvalue+= labelFor + '<br/>';
			   }
			   
			  // attrNamesNeedvalue.push(attribute);
		   }
		});
		
		
		
		if (attrNamesNeedvalue != '')
		{
			DraUtils._showMessage(attrNamesNeedvalue);
			return false;
		}
		else
		{
			var nma = DraUtils._checkAttributePresentOrReadOnly();
			
			if (nma != '')
			{
				var alertMsg = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=attrNotPresentOrreadonlyForUser --><!-- /WSLET -->';
				var confirmMsg = alertMsg.replace('{0}', nma);
				return confirm(confirmMsg);
			}
		}
	}
	
	return true;
	
}

DraUtils._checkAttributePresentOrReadOnly = function()
{
	if (IS_DRA_ENABLED)
	{
		var forAttribute;
		var ele;
		var nonModifiableAttrs = '';
	
		for (var i = 0; i < DRA_DATA.length ; i++)
		{
			var attrRule = DRA_DATA[i];
			forAttribute = attrRule.attribute_name;
			ele = dojo.byId(forAttribute);
			
			if (ele)
			{
				var valueRules = attrRule.rule;
				var currAttrValue = CtrlUtils.getValue(forAttribute);
				for (var u = 0; u < valueRules.length; u++)
				{
					if (currAttrValue == valueRules[u].value)
					{
						var reqAttrs = valueRules[u].req_attributes;
						
						for (var k = 0; k < reqAttrs.length; k++)
						{
							var reqAttr = reqAttrs[k];
							var alias = DraUtils._getAttributeAlias(reqAttr);
							alias = (alias) ? alias : reqAttr;
							var reqAttrEle = dojo.byId(reqAttr);
							
							if (reqAttrEle)
							{
								var isReadOnly = CtrlUtils.isReadOnly(reqAttr);
								
								if (isReadOnly)
								{
									if (nonModifiableAttrs == '')
									{
										nonModifiableAttrs += alias;
									}
									else
									{
										nonModifiableAttrs += ', ' + alias;
									}
								}
							}
							else
							{
								if (nonModifiableAttrs == '')
								{
									nonModifiableAttrs += alias;
								}
								else
								{
									nonModifiableAttrs += ', ' + alias;
								}
							}
						}
					}
				}
			}
		}
		
		return nonModifiableAttrs
	}
}

DraUtils._hasEmptyValue = function(attributeName)
{
	 var attributeValue = CtrlUtils.getValue(attributeName);
	 
	 return attributeValue == '';
}

DraUtils._addToMarkedAttributeList = function(attrElement, markedByAttribute)
{
	var alreadyMarkedList = attrElement.getAttribute('markedBy');
	var toAdd = '|' + markedByAttribute;
	if (alreadyMarkedList)
	{
		if (alreadyMarkedList.indexOf(toAdd) == -1)
		{
			alreadyMarkedList+= toAdd;
		}
		
		attrElement.setAttribute('markedBy', alreadyMarkedList);
	}
	else
	{
		attrElement.setAttribute('markedBy', toAdd);
	}
}

DraUtils._removeFormMarkedAttributeList = function(attrElement, markedByAttribute)
{
	var alreadyMarkedList = attrElement.getAttribute('markedBy');
	var newList = '';
	if (alreadyMarkedList)
	{
		var toRemove = '|' + markedByAttribute;
		if (alreadyMarkedList.indexOf(toRemove) >= 0)
		{
			newList = alreadyMarkedList.replace(toRemove, '');
		}
		else
		{
			newList = alreadyMarkedList;
		}
		
		attrElement.setAttribute('markedBy', newList);
	}
	
	return newList;
}

DraUtils._isReadOnlyAttribute = function(attribute)
{
	var isReadOnly = attribute.getAttribute('isReadOnly');
	
	if (isReadOnly)
	{
		return CtrlUtils.equalsTrue(isReadOnly)
	}
	else
	{
		return false;
	}
}

DraUtils._getAttributeAlias = function(attributeName)
{
	var attrLabelNode = dojo.byId(DraUtils.LABEL + attributeName);
	var attrLabel;
	
	if (attrLabelNode != null)
	{
		var innerText = attrLabelNode.innerHTML;
		var index = innerText.indexOf(':');
		var attrLabel;
		
		if (index != -1)
		{
			if (innerText.indexOf('* ') > -1)
			{
				attrLabel = innerText.substring(2, index);
			}
			else
			{
				attrLabel = innerText.substring(0, index);
			}
		}
	}
	else
	{
		attrLabel = attributeName;
	}
	
	return attrLabel;
}

DraUtils._isForBulkModification = function()
{
	if (typeof forBulkModify != 'undefined')
	{
		return forBulkModify;
	}
	else
	{
		return false;
	}
}
DraUtils._showMessage = function(attrList)
{
	var msg = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='draAttributeRequired' --><!-- /WSLET -->" + attrList;
	PTAlert(msg, 2);
}

dojo.addOnLoad(DraUtils.setupDra);
dojo.addOnLoad(DraUtils.applyDraRulesOnLoad);