/*
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */


 if (!window.EventUtils) alert('TypeFragments requires EventUtils.js');
if (!window.USER_RESOLVERS) alert('TypeFragments requires user name list USER_RESOLVERS from PTDataListBox');
if (!window.getElementsByClassName) alert('TypeFragments requires domUtilities.js');
if (!window.Shybox) alert('TypeFragments requires Shybox.js');
if (!window.token) alert('TypeFragments requires the token');
if (!window.role) alert('TypeFragments requires the role');

function TypeFragments()
{
}

<!-- WSLET CODE=PTAdmin -->
<!-- PARAM NAME=LISTBOX_DEFAULT_VALUE -->
<!-- /WSLET -->

TypeFragments.ANY_VALUE = default_listbox_value;

var uniqueId = 0;

TypeFragments.onValueChange = function()
{
	// override this function in modules that want to know
	// when values go dirty.
};

TypeFragments._saveOriginalTextValue = function(e)
{
	var textbox = EventUtils.getTarget(e);
	if (!textbox._TypeFraments_oldValue)
	{
		textbox._TypeFraments_oldValue = textbox.value;
	}
};

TypeFragments._notifyIfTextValueChanged = function(e)
{
	var textbox = EventUtils.getTarget(e);
	
	if (textbox._TypeFraments_oldValue != textbox.value)
	{
		TypeFragments.onValueChange();
		textbox._TypeFraments_oldValue = textbox.value;
	}
};

TypeFragments.createTextFragment = function(initialValue)
{
	var input = document.createElement('input');
	input.type = 'text';
	input.size = 25;
	
	EventUtils.addEvent(input, 'focus', TypeFragments._saveOriginalTextValue);
	EventUtils.addEvent(input, 'blur', TypeFragments._notifyIfTextValueChanged);

	if (initialValue)
	{
		input.value = initialValue;
	}

	return input;
};

TypeFragments.createShyboxFragment = function(values, initialValue)
{
	// create containing element for positioning context
	var container = document.createDocumentFragment();

	// create the hidden select element used as a shybox
	var list = document.createElement('SELECT');

	for (var i = TypeFragments.getFirstNonAnyValue(values), j = 0; i < values.length; i++, j++)
	{
		list.options[j] = new Option(values[i], values[i]);
	}

	container.appendChild(list);
	Shybox.convertToShybox(
		list,
		initialValue,
		TypeFragments.onValueChange);

	return container;
};

TypeFragments.createShyboxFragmentWithListboxIcon = function(values, initialValue)
{
	var container = TypeFragments.createShyboxFragment(values, initialValue);
	var iconFragment = TypeFragments.createListboxIconFragment();

	for (var i = 0; i < container.childNodes.length; i++)
	{
		if (container.childNodes[i].type == "text")
		{
			container.childNodes[i].id = "inputBox";
		}
	}

	container.appendChild(iconFragment);

	return container;
}

TypeFragments.createShyboxFragmentWithListboxIconWithCustomId = function(values, initialValue, customId)
{
	var container = TypeFragments.createShyboxFragment(values, initialValue);
	if(document.getElementById(customId))
	{
		customId = customId+"#"+(uniqueId++);
	}
	var iconFragment = TypeFragments.createListboxIconFragment(customId);

	for (var i = 0; i < container.childNodes.length; i++)
	{
		if (container.childNodes[i].type == "text")
		{
			container.childNodes[i].id = customId;
		}
	}
	
	container.appendChild(iconFragment);

	return container;
}

TypeFragments.createListboxIconFragment = function(value)
{
	var icon = document.createElement("a");
	var link = document.createElement("img");

	link.title = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='listboxDependencyHelp' --><!-- /WSLET -->";
	link.src = "<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->listbox.gif";
	link.border = "0";

	icon.style.paddingLeft = "5px";
	if (value)
	{
		icon.href = "javascript: TypeFragments.dependentListboxOnclick('"+value+"')";
	}
	else
	{
		icon.href = "javascript: TypeFragments.dependentListboxOnclick()";
	}
	icon.appendChild(link);

	return icon;
}

TypeFragments.dependentListboxOnclick = function()
{
	// This method should be overridden
	alert("TypeFragments.dependentListboxOnclick() should be overridden");
}

TypeFragments.createListboxFragment = function(values, initialValue)
{
	var select = document.createElement('SELECT');
	select.size = 1;
	EventUtils.addEvent(select, 'change', TypeFragments.onValueChange);
	
	var foundInitial = false;

	for (var i = TypeFragments.getFirstNonAnyValue(values), j = 0; i < values.length; i++, j++)
	{
		select.options[j] = new Option(values[i], values[i]);

		if (initialValue == values[i])
		{
			select.options[j].selected = true;
			foundInitial = true;
		}
	}

	if (initialValue && !foundInitial)
	{
		select.options[select.options.length] = new Option(initialValue, initialValue);
	}
	
	return select;
};

TypeFragments.createValueListboxFragment = function(values, initialValue)
{
	var select = document.createElement('select');
	select.size = 1;
	EventUtils.addEvent(select, 'change', TypeFragments.onValueChange);

	var foundInitial = false;

	for (var i = TypeFragments.getFirstNonAnyValue(values), j = 0; i < values.length; i++, j++)
	{
		select.options[j] = new Option(values[i].label, values[i].value);

		if (initialValue == values[i].value)
		{
			select.options[j].selected = true;
			foundInitial = true;
		}
	}

	if (initialValue && !foundInitial)
	{
		select.options[select.options.length] = new Option(initialValue, initialValue);
	}
	
	return select;
};

TypeFragments.createDateFragment = function(values)
{
	return document.createTextNode('<CCM_DATE not yet implemented>');
};

TypeFragments.createUserShyboxFragment = function(initialValue)
{
	var frag = document.createDocumentFragment();

	var select = document.createElement('select');
	// identify user listboxes for later look-up in users_update_OnClick
	select.className = 'userListbox';
		
	if (this.userListboxNum == undefined)
	{
		this.userListboxNum = 0;
	}
	else
	{
		this.userListboxNum++;
	}
	
	select.id = 'userListbox' + this.userListboxNum;
	
	TypeFragments.updateUserListbox(select);
	
	// create containing DIV for positioning context
	frag.appendChild(select);
	Shybox.convertToShybox(
		select,
		initialValue,
		TypeFragments.onValueChange);
	
	if (initialValue)
	{
		Shybox.setValue(select, initialValue);
		SelectUtils.selectValue(select, initialValue, true);
	}
	
	var img = document.createElement('img');
	img.style.cursor = 'pointer';
	frag.appendChild(img);

	img.src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->bb_profile_sm.gif';
	img.alt = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=alt00071 --><!-- /WSLET -->';
	img.align = 'absmiddle';
	img.onclick = function(){TypeFragments.showFavoriteUsersEditor(select);};

	return frag;
};

TypeFragments.getFirstNonAnyValue = function(values)
{
	if (values.length > 0)
	{
		var firstVal = typeof values[0] == 'object' ? values[0][1] : values[0];
		if (firstVal == TypeFragments.ANY_VALUE)
		{
			return 1;
		}
	}

	return 0;
};

TypeFragments.showFavoriteUsersEditor = function(select)
{
	var url = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTWEB_SERVLET --><!-- /WSLET -->' +
			  '?ACTION_FLAG=admin_form' +
			  '&TEMPLATE_FLAG=SubmitCustomUserList' +
			  '&role='  + role + 
			  '&token=' + token;
	window.selectedUserControl = select;
	window.selectedUserControl_changeFocus = TypeFragments.users_fake_changeFocus;

	var w = document.open(url, 'custom_user_list', 'resizable,width=710,height=390', false);
	w.focus();
};

TypeFragments.updateUserListbox = function(lbx)
{
	var currentValue = (lbx._Shybox_textbox)
		? lbx._Shybox_textbox.value
		: null;

	var foundValue = false;
	lbx.length = 0;

	if (!USER_RESOLVERS.resolvers) 
	{
		USER_RESOLVERS.resolvers = new Array();
	}
	
	for (var i = TypeFragments.getFirstNonAnyValue(USER_RESOLVERS.resolvers); i < USER_RESOLVERS.resolvers.length; i++)
	{
		lbx.options[lbx.options.length] = new Option(
			USER_RESOLVERS.resolvers[i][1],
			USER_RESOLVERS.resolvers[i][0]);

		if (currentValue
			&& USER_RESOLVERS.resolvers[i][0] == currentValue)
		{
			foundValue = true;
			lbx.options[lbx.options.length - 1].selected = true;
		}
	}

	// append the previously selected value if it was absent
	if (!foundValue && (currentValue != null))
	{
		lbx.options[lbx.options.length] = new Option(currentValue,currentValue);
	}
};


TypeFragments.users_fake_changeFocus = function(attributeId, markDirty)
{
	var select = document.getElementById(attributeId);
	select._Shybox_textbox.value = select.options[select.selectedIndex].value;
}

// callback function as required by the Favorite Users Dialog (called on "OK")
function users_update_OnClick(list)
{
	// reload data structure
	USER_RESOLVERS.resolvers.length = 0;

	if (list.length == 0)
	{
		USER_RESOLVERS.resolvers = [[TypeFragments.ANY_VALUE, TypeFragments.ANY_VALUE]];
	}
	else
	{
		for (var i = 0; i < list.length; i++)
		{
			USER_RESOLVERS.resolvers[i] = [list[i].uid, list[i].displayName];
		}
	}

	// refresh the user listboxes
	var userListboxes = getElementsByClassName(window.document, 'userListbox');

	for (var i = 0; i < userListboxes.length; i++)
	{
		TypeFragments.updateUserListbox(userListboxes[i]);
	}
}
