/*
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */

if (!window.SOURCE_ATTRIBUTES_DATA) alert('acl_editor.js requires SOURCE_ATTRIBUTES_DATA from PTAdmin');
if (!window.TextboxUtils) alert('acl_editor.js requires TextboxUtils.js');
if (!window.EventUtils) alert('acl_editor.js requires EventUtils.js');
if (!window.StyleUtils) alert('acl_editor.js requires StyleUtils.js');
if (!window.AclUtils) alert('acl_editor.js requires AclUtils.js');
if (!window.Shybox) alert('acl_editor.js requires Shybox.js');
if (!window.AclExport) alert('acl_editor.js requires AclExport.js');
if (!window.AclFilter) alert('acl_editor.js requires AclFilter.js');
if (!window.getNextSiblingElement) alert('acl_editor.js requires domUtilities.js');
if (!window.ScrapeText) alert('acl_editor.js requires ScrapeText.js');

<!-- WSLET CODE=GenAcl -->
<!-- PARAM NAME=JAVASCRIPT_IDENTIFIER VALUE=acl -->
<!-- /WSLET -->

var MORE_ATTRIBUTES = '<<<com.telelogic.pseudoAttr.moreAttributes>>>';
var DIVIDER_VALUE = '<<<com.telelogic.pseudoAttr.divider>>>>';
var DIVIDER_TEXT = '----------';

var gModified = false;

function hasModifications()
{
	return gModified;
}

function setModified()
{
	gModified = true;

	StyleUtils.addClass(
		document.getElementById('save-button'),
		'dirty');
}

// redefine callback from TypeFragments to set this page as dirty when a control is modified
TypeFragments.onValueChange = function()
{
	setModified();
};

/**
 * Functions to read the *visible* values of each logical column. 
 * Each array index is a function to read the column with the same index.
 * Used both in sorting and filtering, which both act on what the user sees
 * (not necessarily the underlying values of the controls.).
 */
var gColumnValueFunctions = [
	function(row)
	{
		return ScrapeText.fromSelects(row.cells[0]);
	},

	function(row)
	{
		return ScrapeText.fromInputs(row.cells[1]) + ScrapeText.fromSelects(row.cells[1]);
	},

	function(row)
	{
		return ScrapeText.fromSelects(row.cells[3]);
	},

	function(row)
	{
		return ScrapeText.fromSelects(row.cells[4]);
	},

	function(row)
	{
		return ScrapeText.fromInputs(row.cells[5]);
	}];

var gAttributeMap = buildAttributeMap();

function buildAttributeMap()
{
	var map = new Object();

	for (var i = 0; i < SOURCE_ATTRIBUTES_DATA.length; i++)
	{
		var attribute = SOURCE_ATTRIBUTES_DATA[i];
		map[attribute.name] = attribute;
	}

	return map;
}

var gCurrentAttributes = buildInitialAttributes();

function buildInitialAttributes()
{
	var attributeNameSet = new Object();

	for (var i = 0; i < window.acl.length; i++)
	{
		var rule = window.acl[i];

		for (var j = 0; j < rule.scope.length; j++)
		{
			if (rule.scope[j].matchAll) continue;
			attributeNameSet[rule.scope[j].attribute] = true;
		}
	}
	
	var attributeNames = new Array();

	for (var attributeName in attributeNameSet)
	{
		attributeNames.push(attributeName);
	}
	
	return attributeNames;
}

function buildAttributeListbox(attributeNames, selectedValue)
{	
	var newListbox = document.createElement('select');
	newListbox.id = "attributeListbox";
	var tempOption;

	// attributes names
	for (var i = 0; i < attributeNames.length; i++)
	{
		var name = attributeNames[i];
		var attribute = window.gAttributeMap[name];

		tempOption = new Option(
			name,
			name);

		newListbox.options[newListbox.options.length] = tempOption;

		// IE ignores 'selected' in constructor sometimes
		if (name == selectedValue)
		{
			tempOption.selected = true;
			newListbox.lastSelectedIndex = newListbox.length - 1;
		}

	}
	
	// 'None (matches all)'
	tempOption = new Option(
		'<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=aclNoAttributes --><!-- /WSLET -->',
		AclExport.ATTRIBUTE_NONE);
	
	newListbox.options[newListbox.options.length] = tempOption;

	// IE ignores 'selected' in constructor sometimes
	if (selectedValue == AclExport.ATTRIBUTE_NONE)
	{
		tempOption.selected = selectedValue == AclExport.ATTRIBUTE_NONE;
		newListbox.lastSelectedIndex = newListbox.length - 1;
	}

	// '-----------'
	newListbox.options[newListbox.options.length] = new Option(
		window.DIVIDER_TEXT,
		window.DIVIDER_VALUE);
	newListbox.options[newListbox.options.length - 1].disabled = true;
	
	// 'More Attributes...'
	newListbox.options[newListbox.options.length] = new Option(
		'<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=moreAttributesAction --><!-- /WSLET -->',
		window.MORE_ATTRIBUTES);
	
	EventUtils.addEvent(newListbox, 'change', attributeSelect_onChange, false);
	EventUtils.addEvent(newListbox, 'click', attributeSelect_onClick, false);

	return newListbox;
}

function removeRow_onClick(e)
{
	var removeLink = EventUtils.getTarget(e);
	var row = getAncestorElementByTagName(removeLink, 'tr');
	
	StyleUtils.addClass(row, 'deleting');
	if (!confirm('<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=confirmDeleteAclRule --><!-- /WSLET -->'))
	{
		StyleUtils.removeClass(row, 'deleting');
		return;
	}

	var userRules = row.parentNode;
	userRules.removeChild(row);
	
	if (userRules.rows.length == 0)
	{
		// show the "blank" rule when there are no others
		$('blank-rule').style.display = '';
	}
	
	var rules = $('user-rules');
	Zebra.stripeBody(rules, 'odd-row');
	AclFilter.updateFilterMessages(rules);
	setModified();
}

function buildRow(fromRule)
{
	// create the new row
	var rowTemplate = $('row-template');
	var newRow = rowTemplate.cloneNode(true);
	newRow.removeAttribute('id');
	
	// the hard way to get the cells since IE can't see newRow.cells until
	// after inserting this node in a dom, but inserting before changing this is too slow.
	var cellCount = 0;
	for (var child = newRow.firstChild; child != null; child = child.nextSibling)
	{
		if (child.tagName == 'TD')
		{
			switch (cellCount)
			{
				case 0:
					var nameCell = child;
					break;
				case 1:
					var valueCell = child;
					break;
				case 2:
					var plusMinusCell = child;
					break;
				case 3:
					var accessCell = child;
					break;
				case 4:
					var permissionCell = child;
					break;
				case 5:
					var usersCell = child;
					break;
				case 6:
					var addDeleteCell = child;
					break;
			}

			++cellCount;
		}
	}

	if (fromRule == null)
	{
		var matchAllScope = [{matchAll: true}];
		buildAttributesAndValues(nameCell, valueCell, plusMinusCell, 0, matchAllScope);
	}
	else
	{
		buildAttributesAndValues(nameCell, valueCell, plusMinusCell, 0, (fromRule.scope) ? fromRule.scope : null);

		SelectUtils.selectValue(accessCell.getElementsByTagName('SELECT')[0], fromRule.access, true);
		SelectUtils.selectValue(permissionCell.getElementsByTagName('SELECT')[0], fromRule.permission, true);
		usersCell.getElementsByTagName('INPUT')[0].value = fromRule.users;

		AclRecentUsers.addRecentUsers(fromRule.users);
	}

	return newRow;
}

function addRow(parent, position, isFirst)
{
	var newRow = buildRow();
	
	parent.insertBefore(
		newRow,
		parent.rows[position] ? parent.rows[position] : null);
	
	// tweak the rest of the layout
	Zebra.stripeBody(parent, 'odd-row');
	AclFilter.updateFilterMessages(parent);
	newRow.getElementsByTagName('select')[0].focus();

	if (!isFirst)
	{
		StyleUtils.addClass(newRow, 'creating');
		window.setTimeout(
			function(){
				StyleUtils.removeClass(newRow, 'creating');
			},
			300);
	}

	setModified();
}

function addRow_onClick(e)
{
	EventUtils.preventDefault(e);

	var addLink = EventUtils.getTarget(e);
	var selectedRow = getAncestorElementByTagName(addLink, 'tr');
	
	addRow(selectedRow.parentNode, selectedRow.sectionRowIndex + 1);
}

function addFirst_onClick(e)
{
	EventUtils.preventDefault(e);
	
	$('blank-rule').style.display='none';

	var userRules = $('user-rules');
	addRow(userRules, 0, true);
}

function users_onFocus(e)
{
	var textbox = EventUtils.getTarget(e);

	textbox.oldValue = textbox.value;
}

function users_onBlur(e)
{
	var textbox = EventUtils.getTarget(e);
	
	if (textbox.oldValue
		&& textbox.oldValue != textbox.value)
	{
		setModified();
	}

	AclRecentUsers.addRecentUsers(textbox.value);
}

function users_onKeyup(e)
{
	var event = (window.event) ? window.event : e;
	var textbox = (window.event)
		? event.srcElement
		: event.target;

	var iKeyCode = event.keyCode;
	
	if (event.ctrlKey) return;

	if ((iKeyCode < 32 && iKeyCode != 13) || (iKeyCode >= 33 && iKeyCode <= 46) || (iKeyCode >= 112 && iKeyCode <= 123))
	{
		return;
	}
	
	AclRecentUsers.complete(textbox);
}

function filter_onSubmit(e)
{
	EventUtils.preventDefault(e);
	
	var filterText = $('filter-string');
	AclFilter.filterRows(filterText.value, gColumnValueFunctions);
	$('show-all-button').disabled = false;
	filterText.select();
	filterText.focus();
}

function showAllButton_onClick(e)
{
	EventUtils.preventDefault(e);
	AclFilter.showAll();
	$('filter-string').focus();

}

function showAllMessage_onClick(e)
{
	EventUtils.preventDefault(e);
	AclFilter.showAll();
}

function hideOrShowPlus(plusMinusCell, forceHide)
{
	var attributeCount = gCurrentAttributes.length;

	var pluses = getElementsByClassName(plusMinusCell, 'attribute-plus-box');

	var visibility = (pluses.length >= attributeCount || forceHide)
		? 'hidden'
		: 'visible';

	for (var i = 0; i < pluses.length; i++)
	{
		pluses[i].style.visibility = visibility;
	}
}

function disableUsedAttributes(attributeCell)
{
	var usedAttributeSet = getUsedAttributeSet(attributeCell);
	disableAttributes(attributeCell, usedAttributeSet);
}

function disableAttributes(attributeCell, usedAttributeSet)
{
	var attributeListboxes = attributeCell.getElementsByTagName('select');
	usedAttributeSet[window.DIVIDER_VALUE] = true;
	
	for (var i = 0; i < attributeListboxes.length; i++)
	{
		SelectUtils.disableValues(
			attributeListboxes[i],
			usedAttributeSet);
	}
}

function getAllattributeListboxes()
{
	var allattributeListboxes = new Array();
	var userRules = $('user-rules');

	for (var i = 0; i < userRules.rows.length; i++)
	{
		var row = userRules.rows[i];
		
		var attributeListboxes = row.cells[0].getElementsByTagName('SELECT');
		
		for (var j = 0; j < attributeListboxes.length; j++)
		{
			allattributeListboxes.push(attributeListboxes[j]);
		}
	}

	return allattributeListboxes;
}

function createArrayFromCurrentListboxValues(listboxes)
{
	var seenSet = new Object();
	seenSet[AclExport.ATTRIBUTE_NONE] = true;
	seenSet[window.MORE_ATTRIBUTES] = true;

	var array = new Array();

	for (var i = 0; i < listboxes.length; i++)
	{
		var listbox = listboxes[i];
		var index = listbox.selectedIndex;
		if (-1 != index)
		{
			var value = listbox.options[index].value;

			if (seenSet[value]) continue;

			array.push(value)
			seenSet[value] = true;
		}
	}
	
	return array;
}

/**
Gets the array elements in the first array, but not in the second.
*/
function findExclusiveElements(first, second)
{
	var exclusives = new Array();

	firstLoop:
	for (var i = 0; i < first.length; i++)
	{
		var firstElement = first[i];

		for (var j = 0; j < second.length; j++)
		{
			var secondElement = second[j];

			if (firstElement == secondElement) continue firstLoop;
		}

		exclusives.push(firstElement);
	}

	return exclusives;
}

function setAttributeSelect(attributeListbox, attributeOpts)
{
	// save off pseudo-attrs
	var noneOpt = attributeListbox.options[attributeListbox.options.length - 3];
	var dividerOpt = attributeListbox.options[attributeListbox.options.length - 2];
	var moreAttributesOpt = attributeListbox.options[attributeListbox.options.length - 1];

	
	// set to the new options
	var selectedIndex = attributeListbox.selectedIndex;
	var currentAttribute = (selectedIndex == -1)
		? null
		: attributeListbox.options[attributeListbox.selectedIndex].value;
	
	SelectUtils.setOptionCopies(attributeListbox, attributeOpts, currentAttribute);

	// restore pseudo-attrs
	attributeListbox.options[attributeListbox.options.length] = noneOpt;
	attributeListbox.options[attributeListbox.options.length] = dividerOpt;
	attributeListbox.options[attributeListbox.options.length] = moreAttributesOpt;

	attributeListbox.selectedIndex = selectedIndex;
}

function setAllAttributeListboxes(newAttributeNames)
{
	var userRules = $('user-rules');

	// clone the new SELECT to replace all the old selects
	for (var i = 0; i < userRules.rows.length; i++)
	{
		var row = userRules.rows[i];
		var nameCell = row.cells[0];
		var attributeListboxes = nameCell.getElementsByTagName('SELECT');
		
		var forceHidePlus = false;
		
		for (var j = 0; j < attributeListboxes.length; j++)
		{
			var oldListbox = attributeListboxes[j];
			var oldValue = SelectUtils.getSelectedValue(oldListbox);
			
			if (oldValue == AclExport.ATTRIBUTE_NONE)
			{
				// don't allow the user to add more attributes is "None" is selected.
				// In this case, this should be the only attribute.
				forceHidePlus = true;
			}
			
			var newListbox = buildAttributeListbox(newAttributeNames, oldValue);
			
			oldListbox.parentNode.replaceChild(newListbox, oldListbox);
		}
		
		disableUsedAttributes(nameCell);
	
		var plusMinusCell = row.cells[2];
		
		hideOrShowPlus(plusMinusCell, forceHidePlus);
	}
}

function favoriteAttributesCallback(chosenNames)
{	
	var attributeListboxes = getAllattributeListboxes();
	var activeNames = createArrayFromCurrentListboxValues(attributeListboxes);

	var newNames = findExclusiveElements(chosenNames, activeNames);

	var totalNames = activeNames.concat(newNames);

	gCurrentAttributes = totalNames;
		
	setAllAttributeListboxes(totalNames);
}

function showAttributeSelectWindow()
{
	var availableAttributes = gCurrentAttributes;

	var url = "<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTWEB_SERVLET --><!-- /WSLET -->" +
		"?ACTION_FLAG=acl_form" +
		"&TEMPLATE_FLAG=AclAttributes" +
		"&role="  + role + 
		"&token=" + token +
		"&ACL_ID=" + acl.id +
		"&SUPP_CHOSEN_ATTRIBUTES=" + escape(availableAttributes.join(' '));
			
	document.open(url, 'acl_attribute_editor', 'resizable,width=600,height=340', false).focus();
}

function attributeSelect_onClick(e)
{
	var target = EventUtils.getTarget(e);
	
	var attributeListbox = (target.tagName == 'OPTION')
		? target.parentNode
		: target;
	
	if (attributeListbox.selectedIndex == -1) return;

	if (attributeListbox.options[attributeListbox.selectedIndex].value == window.MORE_ATTRIBUTES)
	{
		showAttributeSelectWindow();
		return;
	}
}

function hideAllPlusesAndMinuses(cell)
{
	var imgs = cell.getElementsByTagName('img');

	for (var i = 0; i < imgs.length; i++)
	{
		imgs[i].style.visibility = 'hidden';
	}
}

function attributeSelect_onChange(e)
{
	var attributeListbox = EventUtils.getTarget(e);

	var options = attributeListbox.options;
	var selected = attributeListbox.selectedIndex;
	
	if (attributeListbox.options[selected].disabled)
	{
		attributeListbox.selectedIndex = attributeListbox.lastSelectedIndex;
		return;
	}

	var attribute = attributeListbox.options[selected].value;
	
	if (attribute == window.MORE_ATTRIBUTES)
	{
		attributeListbox.selectedIndex = attributeListbox.lastSelectedIndex;
		showAttributeSelectWindow(attributeListbox);
		return;
	}
	else if (attribute == window.DIVIDER_VALUE)
	{
		// assert(should never get here since dividers are disabled)
		return;
	}
	
	var nameDiv = attributeListbox.parentNode;
	var nameCell = nameDiv.parentNode;
	
	var plusMinusCell = nameCell.parentNode.cells[2];

	if (attribute == AclExport.ATTRIBUTE_NONE)
	{
		if (!removeNonAllAttributes(nameCell))
		{
			attributeListbox.selectedIndex = attributeListbox.lastSelectedIndex;
			return;
		}
		hideAllPlusesAndMinuses(plusMinusCell)
	}
	else
	{
		hideOrShowPlus(plusMinusCell);
	}
	
	attributeListbox.lastSelectedIndex = attributeListbox.selectedIndex;
	
	disableUsedAttributes(nameCell);
	
	var valueCell = nameCell.parentNode.cells[1];
	var valueDiv = getChildElementByIndex(
		valueCell,
		getElementIndex(nameDiv));
	drawValueWidget(attribute, valueDiv);

	setModified();
}

function removeNonAllAttributes(attributeCell)
{
	var toRemove = new Array();
	var attributes = attributeCell.getElementsByTagName('select');

	for (var i = 0; i < attributes.length; i++)
	{
		if (SelectUtils.getSelectedValue(attributes[i]) != AclExport.ATTRIBUTE_NONE)
		{
			toRemove.push(i);
		}
	}
	
	if (toRemove.length > 0)
	{
		if (!confirm('<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=removeAllAttributesConfirm --><!-- /WSLET -->'))
		{
			return false;
		}
	}

	for (var i = toRemove.length - 1; i >= 0; i--)
	{
		removeAttributeFromRow(attributeCell.parentNode, toRemove[i])
	}

	return true;
}

function getUsedAttributeSet(nameCell)
{
	var selects = nameCell.getElementsByTagName('select');
	
	var usedAttributes = new Object();

	for (var i = 0; i < selects.length; i++)
	{
		var select = selects[i];
		var option = select.options[select.selectedIndex];

		if (option.value == window.MORE_ATTRIBUTES) continue;
		usedAttributes[option.value] = true;
	}
	
	return usedAttributes;
}

function pickUniqueAttribute(select, usedAttributeSet)
{
	var options = select.options;
	
	if (select.options.length == 3)
	{
		// if the only attributes are all pseudo-attrs (None, divider, More Attrs...)
		// we have to pick None (index 0)
		select.lastSelectedIndex = 0;
		select.selectedIndex = 0;

		return AclExport.ATTRIBUTE_NONE;
	}

	for (var i = 0; i < options.length; i++)
	{
		if (!usedAttributeSet[options[i].value]
				&& options[i].value != AclExport.ATTRIBUTE_NONE)
		{
			options[i].selected = true;

			// the 'select' event listener sets this, but that doesn't fire
			// when selecting programatically. Need to set it here to match that code :(
			select.lastSelectedIndex = select.selectedIndex;

			return options[i].value;
		}
	}

	return select.options[select.selectedIndex].value;
}

function drawValueWidget(attributeName, valueNode, initialValue)
{
	var newValue;

	if (attributeName == window.MORE_ATTRIBUTES)
	{
		// we shouldn't normally get here. Only if the user has no attributes defined in the lifecycle.
		var selectNote = document.createElement('DIV');
		selectNote.className = 'note';
		selectNote.appendChild(document.createTextNode(
			'<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=aclMoreAttributesInfo --><!-- /WSLET -->'));
		newValue = selectNote;
	}
	else
	{
		var attribute = window.gAttributeMap[attributeName];
		
		var type = (attribute)
			? attribute.type
			: 'CCM_STRING';
		
		if (attributeName == AclExport.ATTRIBUTE_NONE)
		{
			type = '{all}';
		}

		switch (type)
		{
			case '{all}':
				newValue = document.createDocumentFragment();
				break;
			case 'CCM_TOGGLE':
				newValue = TypeFragments.createListboxFragment(attribute.values, initialValue);
				break;
			case 'CCM_LISTBOX':
				if (attribute.isDependent)
				{
					newValue = TypeFragments.createShyboxFragmentWithListboxIconWithCustomId(attribute.values, initialValue, attributeName);
				}
				else
				{
					newValue = TypeFragments.createShyboxFragment(attribute.values, initialValue);
				}
				break;
			case 'CCM_VALUELISTBOX':
				newValue = TypeFragments.createValueListboxFragment(attribute.values);
				break;
			case 'CCM_USER':
				newValue = TypeFragments.createUserShyboxFragment(initialValue);
				break;
			case 'relationship':
			default: // all others, e.g. CCM_STRING, CCM_TEXT
				newValue = TypeFragments.createTextFragment(initialValue);
		}
	}

	removeChildren(valueNode);
	valueNode.appendChild(newValue);
}

function selectAttribute(attributeListbox, attribute)
{
	SelectUtils.selectValue(attributeListbox, attribute);

	attributeListbox.lastSelectedIndex
			= attributeListbox.selectedIndex;
}

function buildDefaultAttributesAndValues(nameCell, valueCell, plusMinusCell, position)
{
	var nullEntry = [new Object()];
	buildAttributesAndValues(nameCell, valueCell, plusMinusCell, position, nullEntry);
}

function buildAttributesAndValues(nameCell, valueCell, plusMinusCell, position, attributeValuePairs)
{
	var nameRows = document.createDocumentFragment();
	var valueRows = document.createDocumentFragment();
	var plusMinusRows = document.createDocumentFragment();

	var usedAttributeSet = getUsedAttributeSet(nameCell);
	
	var onlyOneAttribute = attributeValuePairs.length == 1
		&& nameCell.childNodes.length == 0;
	
	var forceHiddenPlus;

	for (var i = 0; i < attributeValuePairs.length; i++)
	{
		// create the attribute name cell
		var nameDiv = document.createElement('DIV');
		nameDiv.className = 'attribute-value-cell';

		var attributeListbox = buildAttributeListbox(gCurrentAttributes);
		attributeListbox.valueDiv = valueDiv;
		nameDiv.appendChild(attributeListbox);

		// create the attribute value cell (based on type of attribute)
		var valueDiv = document.createElement('DIV');
		valueDiv.className = 'attribute-value-cell';

		// create the + - buttons cells
		var plusMinusDiv = document.createElement('DIV');
		plusMinusDiv.className = 'plus-minus-cell';

		var minusImg = document.createElement('img');
		if (onlyOneAttribute) minusImg.style.visibility = 'hidden';
		minusImg.src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->minus_box.gif';
		minusImg.className = 'attribute-minus-box';
		minusImg.title = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=aclRemoveAttributeHelp --><!-- /WSLET -->'
		minusImg.alt = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=aclRemoveAttributeHelp --><!-- /WSLET -->'
		EventUtils.addEvent(minusImg, 'click', removeAttribute_onClick);
		plusMinusDiv.appendChild(minusImg);

		plusMinusDiv.appendChild(document.createTextNode(' '));
		
		var plusImg = document.createElement('IMG');
		plusImg.src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->plus_box.gif';
		plusImg.className = 'attribute-plus-box';
		plusImg.title = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=aclAddAttributeHelp --><!-- /WSLET -->';
		plusImg.alt = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=aclAddAttributeHelp --><!-- /WSLET -->';
		EventUtils.addEvent(plusImg, 'click', addAttribute_onClick);
		
		plusMinusDiv.appendChild(plusImg);
	
		plusImg.complimentMinus = minusImg;
		
		var currentPair = attributeValuePairs[i];
		
		if (currentPair.matchAll)
		{
			// not really a pair in this case, just an object to indicates we should always match (i.e., no attributes)
			selectAttribute(attributeListbox, AclExport.ATTRIBUTE_NONE);
			drawValueWidget(AclExport.ATTRIBUTE_NONE, valueDiv);
			forceHiddenPlus = true;
		}
		else if (currentPair.attribute)
		{
			// use given attributes (on load)
			selectAttribute(attributeListbox, currentPair.attribute);
			usedAttributeSet[currentPair.attribute] = true;
			drawValueWidget(currentPair.attribute, valueDiv, currentPair.value);
		}
		else
		{
			// select unique attribute name (after clicking plus)
			var uniqueAttribute = pickUniqueAttribute(attributeListbox, usedAttributeSet);
			drawValueWidget(uniqueAttribute, valueDiv);
			if (uniqueAttribute != window.MORE_ATTRIBUTES) usedAttributeSet[uniqueAttribute] = true;
		}

		// save the rows
		nameRows.appendChild(nameDiv);
		valueRows.appendChild(valueDiv);
		plusMinusRows.appendChild(plusMinusDiv);
	}

	// add the rows to their tables
	nameCell.insertBefore(nameRows, nameCell.childNodes[position] ? nameCell.childNodes[position] : null);
	valueCell.insertBefore(valueRows, valueCell.childNodes[position] ? valueCell.childNodes[position] : null);
	plusMinusCell.insertBefore(plusMinusRows, plusMinusCell.childNodes[position] ? plusMinusCell.childNodes[position] : null);

	// must disable names after adding to the table so that all the
	// rows can be searched and grayed, not just new ones.
	disableAttributes(nameCell, usedAttributeSet);
	hideOrShowPlus(plusMinusCell, forceHiddenPlus);
}

function removeAttributeFromRow(row, attributeIndex)
{
	function deleteChildElementByIndex(node, index)
	{
		node.removeChild(
			getChildElementByIndex(node, index));
	}

	// attr name
	deleteChildElementByIndex(row.cells[0], attributeIndex);
	// attr value
	deleteChildElementByIndex(row.cells[1], attributeIndex);
	// +/- controls
	deleteChildElementByIndex(row.cells[2], attributeIndex);
}

function removeAttribute_onClick(e)
{
	var minusImg = (window.event) ? window.event.srcElement : e.target;
	var minusDiv = minusImg.parentNode;
	
	var row = minusDiv.parentNode.parentNode;

	// +/- widgets are in their own cell. Every pair is in a DIV and each DIV
	// corresponds to similar DIVs in the name and value cells. These DIV tuples
	// triples make make up an attribute, with every DIV having the same element index
	// in its owning cell. By getting the DIVs index, we know which name and value
	// DIVs to remove in the other cells.
	var attributeIndex = getElementIndex(minusDiv);
	removeAttributeFromRow(row, attributeIndex);
	
	// if there are two images left, they are the last plus and minus.
	// Hide the last minus.
	var controlImgs = row.cells[2].getElementsByTagName('IMG');

	if (controlImgs.length == 2)
	{
		controlImgs[0].style.visibility = 'hidden';
	}

	var nameCell = row.cells[0];
	var plusMinusCell = row.cells[2];
	disableUsedAttributes(nameCell);
	hideOrShowPlus(plusMinusCell);
	setModified();
}

function addAttribute_onClick(e)
{
	var plusImg = (window.event) ? window.event.srcElement : e.target;
	
	plusImg.complimentMinus.style.visibility = 'visible';

	var plusDiv = plusImg.parentNode;
	var index = getElementIndex(plusDiv);
	
	var rule = plusDiv.parentNode.parentNode;
	
	buildDefaultAttributesAndValues(rule.cells[0], rule.cells[1], rule.cells[2], index + 1);
	setModified();
}

function initRules(acl)
{
	$('blank-rule').style.display = (window.acl.length == 0)
		? ''
		: 'none';

	var userRules = $('user-rules');
	
	var rows = document.createDocumentFragment();

	for (var i = 0; i < window.acl.length; i++)
	{
		rows.appendChild(
			buildRow(window.acl[i]));
	}
	
	SelectUtils.selectValue($('default-select'), window.acl.defaultRule, true);
	
	Zebra.stripeBody(rows, 'odd-row');

	userRules.appendChild(rows);
	AclFilter.updateFilterMessages($('user-rules'));
}

function showAclManager()
{
	var form = document.getElementById('cancel-form');

	form.token.value = token;
	form.role.value = role;

	form.submit();
}

function save_onClick(e)
{
	var userRules = $('user-rules');
	var form = AclExport.convertToForm(userRules.rows);

	form.method = 'post';
	form.action = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTADMINACTION_SERVLET --><!-- /WSLET -->';
	form.target = 'pt_status';

	form.appendChild(AclExport.buildInput('ACTION_FLAG', 'save_acl'));
	form.appendChild(AclExport.buildInput('token', token));
	form.appendChild(AclExport.buildInput('role', role));
	form.appendChild(AclExport.buildInput('ACL_ID', acl.id));
	
	form.appendChild(AclExport.buildInput('CCM_CALLBACK', 'window.parent.workspace.showAclManager();'));
	
	var defaultValue = $('default-rule').getElementsByTagName('SELECT')[0].value;
	form.appendChild(AclExport.buildInput('default_rule', defaultValue));
	
	// a dynamically created form must be in the DOM in order to 'submit'
	userRules.appendChild(form);
	form.submit();
	userRules.removeChild(form);
}

function cancel_onClick(e)
{
	showAclManager();
}

function back_onClick(e)
{
	EventUtils.preventDefault(e);
	
	if (hasModifications()
		&& !confirm('<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=err00074 --><!-- /WSLET -->'))
	{
		return;
	}

	showAclManager();
}

function initHeader()
{
	$('header').appendChild(
		document.createTextNode(acl.displayName));

	var message;

	if (acl.id == 'Tasks')
	{
		message = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=defaultAclTaskRule --><!-- /WSLET -->';
	}
	else if (acl.id == 'Objects')
	{
		message = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=defaultAclObjectRule --><!-- /WSLET -->';
	}
	else
	{
		message = '<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=defaultAclCrRule --><!-- /WSLET -->';
	}

	$('default-description').appendChild(
		document.createTextNode(message));
	
	
}

function clearStatus()
{
	if (window.parent.pt_status)
	{
		var message = window.parent.pt_status.document.getElementById('message');
		if (message) removeChildren(message);
	}
}

function init_onLoad(e)
{
	var startTime = new Date().getTime();
	
	clearStatus();
	initHeader();

	EventUtils.addEvent(
		$('filter-form'),
		'submit',
		filter_onSubmit);

	EventUtils.addEvent(
		$('first-rule'),
		'click',
		addFirst_onClick);

	EventUtils.addEvent(
		$('show-all-button'),
		'click',
		showAllButton_onClick);

	EventUtils.addEvent(
		$('show-all-message'),
		'click',
		showAllMessage_onClick);
	
	EventUtils.addEvent(
		$('save-button'),
		'click',
		save_onClick);

	EventUtils.addEvent(
		$('cancel-button'),
		'click',
		cancel_onClick);
	
	EventUtils.addEvent(
		$('back-link'),
		'click',
		back_onClick);
	
	initRules(window.acl);
	
	new TableSorter($('user-rules'), gColumnValueFunctions, 'odd-row');		

	var endTime = new Date().getTime();

	document.getElementById('filter-string').focus();
}

TypeFragments.dependentListboxOnclick = function(value)
{
	window.field = document.getElementById(value);
	
	var val;
	var temp = new Array();
	
	temp = value.split('#');
	val = temp[0];
	
	//var attributeListbox = document.getElementById("attributeListbox");
	//var attribute = attributeListbox.options[attributeListbox.selectedIndex].value;
	//attribute = window.gAttributeMap[attribute];

	var url = "<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTWEB_SERVLET --><!-- /WSLET -->" +
	      "?ACTION_FLAG=dependent_listbox_form" +
		  "&TEMPLATE_FLAG=QueryDependentListbox" +
		  "&attributeName=" + escape(val) +
		  "&SUPP_attributeName=" + escape(val) +
		  "&SUPP_attributeLabel=" + escape(val) +
		  "&role=" + escape(role) +
          "&token=" + token;

    document.open(url, "listboxDependent", 'resizable,scrollbars,width=700,height=300').focus();
}

EventUtils.addEvent(window, 'load', init_onLoad, false);