/**
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */

var frame;
var returnURL = window.location;
var dlgWindow;
var dialog;
var utilityDialog;
var canRefreshCr = false;

function showPickerDialog()
{
	var dlg = dijit.byId("oauthFriendsDialog");
	dlg.show();
	resetListBoxes();
	populateFields();
	clearErrorSection();
	canRefreshCr = false;
	return;
}

function hidePickerDialog()
{
	var dlg = dijit.byId("oauthFriendsDialog");
	dlg.hide();
	
	if (canRefreshCr)
	{	
		refresh();
	}
	
	return;
}

function populateFields()
{
	displayWaitingSection();
	OAuthFriendsUtilsAjax.getOAuthFriendsAsJson(token, {callback: callbackPopulateFields, errorHandler: dwrErrorHandler});
}

function callbackPopulateFields(data)
{
	dojo.byId("oauth-friends-list").disabled = false;
	SelectUtils.clearList("oauth-friends-list");
	hideWaitingSection();
	SelectUtils.addOption("oauth-friends-list", "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='selectAnOslcFriend' --><!-- /WSLET -->", "-1", true);
	
	for (var i = 0; i < data.length; i++)
	{
		SelectUtils.addOption("oauth-friends-list", data[i].displayName, data[i].rootservices, false);
	}
}

function processOnSelect()
{
	var friendElem = dojo.byId("oauth-friends-list");
	var index = friendElem.selectedIndex;
	resetListBoxes();
	displayWaitingSection();
	
	if (!isAValidOption(friendElem.options[index]))
	{
		hideWaitingSection();
		return;
	}
	
	var rootServiceUri = friendElem.options[index].value;
	var serviceProviderName = friendElem.options[index].text;
	OAuthFriendsUtilsAjax.getServiceCatalogFromAProvider(token, serviceProviderName, rootServiceUri, {callback: callbackprocessOnSelect, errorHandler: dwrErrorHandler});
}

function callbackprocessOnSelect(data)
{
	var success = false;
	hideWaitingSection();
	
	if (data && data[0] && data[0].needAuthentication)
	{
		postMessageProtocol(data[0].authUrl);
		return;
	}
	
	if (success)
	{
		var friendElem = dojo.byId("oauth-friends-list");
		var index = friendElem.selectedIndex;
		
		var rootServiceUri = friendElem.options[index].value;
		var serviceProviderName = friendElem.options[index].text;
		
		OAuthFriendsUtilsAjax.getServiceCatalogFromAProvider(token, serviceProviderName, rootServiceUri, callbackprocessOnSelect);
	}
	
	dojo.byId("oauth-friends-services").disabled = false;
	SelectUtils.clearList("oauth-friends-services");
	
	for (var i = 0; i < data.length; i++)
	{
		SelectUtils.addOption("oauth-friends-services", data[i].serviceName, data[i].serviceUri, false);
	}
}


function processServiceOnSelect()
{
	displayWaitingSection();
	var serviceElem = dojo.byId("oauth-friends-services");
	var index = serviceElem.selectedIndex;
	
	if (!isAValidOption(serviceElem.options[index]))
	{
		hideWaitingSection();
		return;
	}
	
	var mainServiceProvider = dojo.byId("oauth-friends-list");
	
	var rootServicesUri = mainServiceProvider.options[mainServiceProvider.selectedIndex].value;
	
	if (isAValidOption(serviceElem.options[index]))
	{
		var serviceName = serviceElem.options[index].text;
		var serviceUri = serviceElem.options[index].value;
		
		OAuthFriendsUtilsAjax.getServicesFromAProvider(token, serviceName, serviceUri, rootServicesUri, {callback: serviceOnSelectCallback, errorHandler: dwrErrorHandler});
	}
}

function serviceOnSelectCallback(data)
{
	hideWaitingSection();
	
	dojo.byId("existing-options").disabled = false;
	dojo.byId("create-options").disabled = false;
	
	SelectUtils.clearList("existing-options");
	SelectUtils.clearList("create-options");
	var optionAttr;
	
	for (var i = 0; i < data.length; i++)
	{
		var id = (data[i].dialogForSelection) ? "existing-options" : "create-options";
		optionAttr = SelectUtils.addOption(id, data[i].dialogName, data[i].dialogUri, data[i].dialogDefault);
		optionAttr.setAttribute("frameHeight", data[i].dialogHeight);
		optionAttr.setAttribute("frameWidth", data[i].dialogWidth);
	}
}

function dwrErrorHandler(errorMessage)
{
	hideWaitingSection();
	showError(errorMessage);
}

function showError(errMsg)
{
	showMessage(errMsg, true);
}

function showMessage(message, isError)
{
	var errorSection = dojo.byId("error-section");
	var erroMessageSection = dojo.byId("errorMessageSection");
	var imagePath = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->';
	var massageIconImg = dojo.byId('message-icon');
	erroMessageSection.innerHTML = message;
	
	if (isError)
	{
		erroMessageSection.style.color = "red";
		massageIconImg.src = imagePath + "ws_erroricon_sm.gif";
	}
	else
	{
		massageIconImg.src = imagePath + "ws_successicon_sm.gif";
	}
	
	errorSection.style.display = "";
}

function clearErrorSection()
{
	var errorSection = dojo.byId("error-section");
	var erroMessageSection = dojo.byId("errorMessageSection");
	erroMessageSection.innerHTML = "";
	errorSection.style.display = "none";
}

function showServiceDialog()
{
	var operationElems = document.getElementsByName("link-operation");
	var operationType = "select";
	
	for (var i = 0; i < operationElems.length; i++)
	{
		if (operationElems[i].checked)
		{
			operationType = operationElems[i].value;
			break;
		}
	}
	
	if (operationType == "select")
	{
		showDialog(dojo.byId("existing-options"));
	}
	else
	{
		showDialog(dojo.byId("create-options"));
	}
}


function showDialog(listElem)
{
	var listOptions = listElem.options;
	
	var index = listOptions.selectedIndex;
	var selectedOption = listOptions[index];
	
	if (!isAValidOption(selectedOption))
	{
		alert("<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='selectAValidOption' --><!-- /WSLET -->");
		return;
	}
	
	var frameHeight = selectedOption.getAttribute("frameHeight");
	var frameWidth = selectedOption.getAttribute("frameWidth");
	
	var dialogUrl = selectedOption.value + "#oslc-postMessage-1.0"
	
	frame = document.createElement('iframe');
	frame.src = dialogUrl;
	frame.width = frameWidth; 
	frame.height = frameHeight;
	frame.scrolling = "yes";
	displayUtilityDialog(frame, selectedOption.text);
}


function displayWindow(frameContent)
{
	if (!dialog) dialog = new dijit.Dialog(); 
	dialog.setContent(frame); 
	dialog.show();
}

function displayUtilityDialog(frameContent, dialogName)
{
	utilityDialog = new dijit.Dialog({title: dialogName});
	utilityDialog.setContent(frameContent); 
	utilityDialog.show();
}

function closeWindow(frameWindow)
{
	dialog.hide();
}

function postMessageProtocol(dialogURL)
{
   dialogURL += '#oslc-postMessage-1.0';
   InitEventListener();
   frame = document.createElement('iframe');
   frame.src = dialogURL;
   frame.width = 450; 
   frame.height = 300;
   displayWindow(frame);
}

function eventListener(e)
{
	var HEADER = "oslc-response:";
    if (e.source == frame.contentWindow && e.data.indexOf(HEADER) == 0)
    {
       closeUtilityDialog();
       handleMessage(e.data.substr(HEADER.length));
    }
}

function handleMessage(msg)
{
	var data = dojo.fromJson(msg);
	var results = data['oslc_cm:results'];
	var linkUrl;
	var linkLable;
	
	if (results.length == 0)
	{
		return;
	}
	
	displayWaitingSection();
	for (var i = 0; i < results.length; i++)
	{
		linkUrl = results[i]['rdf:resource'];
		linkLable = results[i]['oslc_cm:label'];
	}
	
	var crId = CtrlUtils.getValue('problem_number');
	OAuthFriendsUtilsAjax.saveLink(token, crId, linkLable, linkUrl, {callback: handleMessageCallback, errorHandler: dwrErrorHandler});
}

function handleMessageCallback(data)
{
	canRefreshCr = true;
	clearErrorSection();
	createBacklinkAtProviderEnd(data);
	closeUtilityDialog();
	return;
}

function createBacklinkAtProviderEnd(linkUri)
{
	//displayWaitingSection();
	var crId = CtrlUtils.getValue('problem_number');
	dojo.xhrGet({
	    url:"<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=BASE_PATH --><!-- /WSLET -->/backlink.do",
	    handleAs: "json",
	    content: {"token": token, "targetUri": linkUri, "crId": crId},
	    headers: {"OSLC-Core-Version":"2.0", "Accept":"application/json"},
	    load: loadOnGet,
	    error: errorOnGet
	    });
}

function loadOnGet(data)
{
	hideWaitingSection();
	hidePickerDialog();
}

function errorOnGet(err, ioArgs)
{
	hideWaitingSection();
	var errMsg = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='unableCratingBacklink' --><!-- /WSLET -->";
	showError(errMsg);
}

function closeUtilityDialog()
{
	if (utilityDialog) utilityDialog.hide();
}

function windowNameProtocol(dialogURL) 
{
	/// (5) // Step #1: create iframe with fragment to indicate protocol
	// Step #2: set the iframe's window.name to indicate the Return URL
	var ie = window.navigator.userAgent.indexOf("MSIE"); 
	if (ie > 0) 
	{
		frame = document.createElement('<iframe id="aFrame" name=\'' + returnURL + '\'>');
	}
	else
	{
		frame = document.createElement('iframe'); 
		frame.name = returnURL; 
	}
	
	frame.src = dialogURL + '#oslc-core-windowName-1.0'; 
	frame.width = 450; 
	frame.height = 300; 
	
	displayWindow(frame); 
	
	// Step #3: listen for onload events on the iframe
	if (ie > 0) 
	{ 
		frame.attachEvent("onload", onFrameLoaded); 
	}
	else 
	{ 
		frame.onload = onFrameLoaded; 
	}
}

function InitEventListener()
{
	if (window.addEventListener) {
		window.addEventListener ("message", eventListener, false);
    }
    else {
        if (window.attachEvent) {   // IE before version 9
        	window.attachEvent("onmessage", eventListener);
        }
    }
}

function isAValidOption(option)
{
	if (option.value != "-1")
	{
		if (option.value == "")
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	return false;
}

function addDefaultOption()
{
	var option = new Option();
	option.text = "";
	option.value = "-1";
	
	return option;
}

function displayWaitingSection()
{
	clearErrorSection();
	var warningElem = dojo.byId("waiting-section");
	warningElem.style.display = "";
}

function hideWaitingSection()
{
	var warningElem = dojo.byId("waiting-section");
	warningElem.style.display = "none";
}

function resetListBoxes()
{
	var listboxes = ["existing-options", "create-options", "oauth-friends-services"];
 
	for (var i = 0; i < listboxes.length; i++)
	{
		dojo.byId(listboxes[i]).options.length = 0;
		dojo.byId(listboxes[i]).options[0] =  addDefaultOption();
		dojo.byId(listboxes[i]).disabled = true;
	}
}

function oauthSuccess()
{
	dialog.hide();
	processOnSelect();
}

function removeLink(attr, label, url)
{
	var messageRow = dojo.byId("message-row");
	messageRow.style.display = "none";
	
	var errorMsgRow = dojo.byId("error-message-row");
	var errorMsgCnt = dojo.byId("error-message-content");
	
	errorMsgCnt.innerHTML = "";
	errorMsgRow.style.display = 'none';
	
	var confirmMessage = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alertRemoveLinkMsg' --><!-- /WSLET -->";
	if (!confirm(confirmMessage))
	{
		return;
	}
	
	var crId = CtrlUtils.getValue('problem_number');
	var messageRow = dojo.byId("message-row");
	messageRow.style.display = "block";
	OAuthFriendsUtilsAjax.removeLink(token, attr, crId, label, url, {callback: removeLinkCallBack, errorHandler: removeLinkErrorHandler});
}

function removeLinkCallBack(message)
{
	//displayWaitingSection();
	var crId = CtrlUtils.getValue('problem_number');
	dojo.xhrGet({
	    url:"<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=BASE_PATH --><!-- /WSLET -->/removeBacklink.do",
	    handleAs: "json",
	    content: {"token": token, "targetUri": message, "crId": crId},
	    headers: {"OSLC-Core-Version":"2.0", "Accept":"application/json"},
	    load: loadOnSuccess,
	    error: handleError
	    });
}

function removeLinkErrorHandler(errorMessage)
{
	var messageRow = dojo.byId("message-row");
	messageRow.style.display = "none";
	
	var errorMsgRow = dojo.byId("error-message-row");
	var errorMsgCnt = dojo.byId("error-message-content");
	
	errorMsgCnt.innerHTML = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='unableRemovinglinkInChange' --><!-- /WSLET -->";
	errorMsgRow.style.color = "red";
	errorMsgRow.style.display = 'block';
}

function loadOnSuccess(message)
{
	var messageRow = dojo.byId("message-row");
	messageRow.style.display = "none";
	refresh();
}

function handleError(errorMessage, ioArgs)
{
	var messageRow = dojo.byId("message-row");
	messageRow.style.display = "none";
	
	var errorMsgRow = dojo.byId("error-message-row");
	var errorMsgCnt = dojo.byId("error-message-content");
	
	errorMsgCnt.innerHTML = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='unableRemovinglinkAtOtherTool' --><!-- /WSLET -->";
	errorMsgRow.style.color = "red";
	errorMsgRow.style.display = 'block';
	
}

dojo.addOnLoad(InitEventListener);