<!-- ----------------------------------------------------------------- -->
<!-- associated_tasks_control_report.js -->

//<!-- WSLET CODE=PTInclude -->
//<!-- PARAM NAME=getElementStyle.js -->
//<!-- /WSLET -->

/**
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */
 
dojo.require("dojo.fx");
dojo.require("dojox.fx.easing");

//use 'block' with IE since it doesn't understand 'table-row'
var tableRowStyle = (document.all)
	? 'block'
	: 'table-row';

function setIndicatorOpenState(row, open)
{
	var imgs = row.getElementsByTagName('img');

	for (var i = 0; i < imgs.length; ++i)
	{
		if (open)
		{
			if (imgs[i].src.lastIndexOf('down_chevron.gif') >= 0)
			{
				// IE hack. If the currentTarget of the event is wrapped in a link, IE sometimes erases the IMG.
				// Swapping the image "later"--after the event is done--prevents the problem.
				window.setTimeout(function(){ imgs[i].src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->up_chevron.gif'; }, 1);
				return;
			}
		}
		else
		{
			if (imgs[i].src.lastIndexOf('up_chevron.gif') >= 0)
			{
				window.setTimeout(function(){ imgs[i].src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->down_chevron.gif'; }, 1);
				return;
			}
		}
	}
}

function setRowVisibility(row, show)
{
	// can't animate tables, so it's wrapped in a div
	var detailContainer = row.getElementsByTagName('div')[0];	
	
	if (show)
	{
		row.style.display = tableRowStyle;
		detailContainer.style.display = "none";
		
		
		dojo.fx.wipeIn(
			{
				node: detailContainer,
				duration: 300,
				easing: dojox.fx.easing.cubicOut,
				onEnd: function(){detailContainer.style.height = 'auto';}
			}).play();
	}
	else
	{
		dojo.fx.wipeOut(
			{
				node: detailContainer,
				duration: 300,
				easing: dojox.fx.easing.cubicOut
			}).play();
	}
}

function setNextRowVisibility(current, show)
{
	var next = current;

	while (null != (next = next.nextSibling))
	{
		if (next.nodeType == 1
			&& 'TR' == next.tagName)
		{
			setRowVisibility(next, show);
			return;
		}
	}
}

function insertLoadingMessage(row, taskNumber)
{
	var tr = document.createElement('TR');

	var td = document.createElement('TD');
	td.colSpan = '5';
	td.style.textAlign = 'center';
	tr.appendChild(td);

	var spinner = document.createElement('img');
	spinner.src = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTIMAGES_PATH --><!-- /WSLET -->spinningWait13px.gif';
	spinner.alt = '';
	spinner.style.height = '13px';
	spinner.style.width = '13px';
	spinner.style.marginRight = '.5em';
	spinner.style.verticalAlign = 'middle';
	td.appendChild(spinner);

	var message = document.createTextNode('<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE=loadingTask --><!-- /WSLET -->');
	td.appendChild(message);

	row.parentNode.insertBefore(tr, row);

	return tr;
}

function expandTaskDetails(row)
{
	if (row.expanded)
	{
		return;
	}

	row.expanded = true;
	
	if (row.loaded)
	{
		expandLoadedTaskDetails(row);
		return;
	}
	
	// else load it
	row.loaded = true;
	
	var taskNumber = row.getAttribute('taskNumber');
	
	var reportUrl = '<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTACTION_SERVLET --><!-- /WSLET -->'
		+ '?token=' + encodeURIComponent(window.token)
		+ '&ACTION_FLAG=immediate_report'
		+ '&CHOSEN_REPORT=Associated+Objects+Control+Report'
		+ '&empty_results_template=NoAssociatedObjectsBanner'
		+ '&max_query_template=QueryLimitExceed'
		+ '&instance=' + encodeURIComponent(row.getAttribute('instance'))
		+ '&task_number=' + encodeURIComponent(taskNumber)
		+ '&save_report=false'
		+ '&QUERY_DBS=' + encodeURIComponent(row.getAttribute('fromDBPath'));
	
	var detailRow = getNextSiblingElement(row);
	var objectReport = getElementsByClassName(detailRow, 'object-report')[0];
	
	var loadingRow = insertLoadingMessage(row, taskNumber);
	dojo.style(row, {display: "none"});

	dojo.xhrGet({
		url: reportUrl,
		error: function(error, args)
		{
			objectReport.innerText = error.message;
			
			dojo.style(row, {display: tableRowStyle});
			removeNode(loadingRow);
			
			expandLoadedTaskDetails(row);
		},
		load: function(data)
		{
			objectReport.innerHTML = data;			
			
			dojo.style(row, {display: tableRowStyle});
			removeNode(loadingRow);

			expandLoadedTaskDetails(row);
		}
	});
}

function removeNode(node){
	if (node && node.parentNode)
	{
		// return a ref to the removed child
		return node.parentNode.removeChild(node);
	}
}

function expandLoadedTaskDetails(row)
{
	setIndicatorOpenState(row, true);
	setNextRowVisibility(row, true);
}

function collapseTaskDetails(row)
{
	if (row.collapsed)
	{
		return;
	}

	row.expanded = false;

	setIndicatorOpenState(row, false);
	setNextRowVisibility(row, false);
}

/**
 * Hides and shows the next row after this one. When showing the row, the border for
 * this one is hidden so that they look like one big row.
 * Also changes the open/closed indicator image.
 */
function toggleTaskDetails(row)
{	
	if (row.expanded)
	{
		collapseTaskDetails(row);
	}
	else
	{
		expandTaskDetails(row);
	}
}

function expandAllTaskDetails()
{
	var taskTable = document.getElementById('associated-tasks-control');

	var taskRows = getElementsByClassName(taskTable, 'clickable-row');

	for (var i = 0; i < taskRows.length; ++i)
	{
		expandTaskDetails(taskRows[i]);
	}

}

function collapseAllTaskDetails()
{
	var taskTable = document.getElementById('associated-tasks-control');

	var taskRows = getElementsByClassName(taskTable, 'clickable-row');

	for (var i = 0; i < taskRows.length; ++i)
	{
		collapseTaskDetails(taskRows[i]);
	}
}

// insert the task expand/collapse functions into the list of all expand functions for this window
if (!window.expandFunctions) {window.expandFunctions = new Array();}
if (!window.collapseFunctions) {window.collapseFunctions = new Array();}
window.expandFunctions.push(expandAllTaskDetails);
window.collapseFunctions.push(collapseAllTaskDetails);

<!-- END associated_tasks_control_report.js -->
<!-- ----------------------------------------------------------------- -->