/**
 * adhocRelReport.js
 * It consists of DOJO RELATED FUNCTIONS FOR DIALOG 'relationship_information_dialog.custom'.
 * Also contains methods invoked by format_edit_dflt.html for adhoc relational formats
 */

/**
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */
 
/**
 * Shows relationship information dialog
 */
function showRelationshipInfoDialog()
{
	dojo.byId("relationshipLbl").style.display = "";
	dojo.byId("relationships").style.display = "";
	dojo.byId("relationships").value = "";
	dojo.byId("cancelBtn").style.display = "";
	dojo.byId("okBtn").style.display = "";
	dijit.byId("relationshipInfoDialog").show();

	if (typeof isDojoDialogOpen != "undefined")
	{
		isDojoDialogOpen = true;
	}

	if (getNavFrame().grayOutQueryFrame)
	{
		getNavFrame().grayOutQueryFrame(true);
	}
}

/**
 * Hide the relationship dojo dialog 
 */
function hideRelationshipInfoDialog()
{
	dijit.byId("relationshipInfoDialog").hide();

	if (typeof isDojoDialogOpen != "undefined")
	{
		isDojoDialogOpen = false;
	}

	if (getNavFrame().grayOutQueryFrame)
	{
		getNavFrame().grayOutQueryFrame(false);
	}
}

/**
 * Load relationships in the Relationship Information dojo dialog
 */
function loadRelationships(listControl, type)
{
	listControl.length = 0;
	
	var list = null;

	if (type == "ALL")
	{
		list = RELATION_ATTRIBUTES;
	}
	else if (type == "OBJECT")
	{
		list = relationshipsForObject;
	}
	else if (type == "TASK")
	{
		list = relationshipsForTask;
	}
	else
	{
		list = relationshipsForCR;
	}
	
	if (list != null)
	{ 
		for (var index = 0; index < list.length; index++)
		{
			listControl.options[listControl.length] = new Option(list[index].alias, list[index].name);
		}
	}

	if (type == "ALL")
	{
		var validRelationship = false;
		for (var index = 0; index < list.length; index++)
		{
			if (listControl.options[index].value == getRelationType())
			{
				validRelationship = true;
				listControl.options[index].selected=true;
			}
		}

		if (!validRelationship)
		{
			listControl.options[listControl.length] = new Option(fetchCurrentReportFromMainReport().relationAlias, getRelationType())
			listControl.options[index].selected=true;
		}
	}
}

function validateRelationships(relRptDataStructure)
{
	var list = RELATION_ATTRIBUTES;
	
	for (var subReportIndex = 0; subReportIndex < relRptDataStructure.sub_reports.length; subReportIndex++)
	{
		var isValidRelationship = false;

		for (var index = 0; index < list.length; index++)
		{
			if (list[index].name == relRptDataStructure.sub_reports[subReportIndex].relation)
			{
				isValidRelationship = true;
			}
		}

		if (!isValidRelationship)
		{
			var formatter = new MessageFormat('<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='invalidRelationship' --><!-- /WSLET -->');
			alert(formatter.format(new Array(relRptDataStructure.sub_reports[subReportIndex].relationAlias))); 
			return false;
		}
	}

	return true;
}

function setupRelationships()
{
	if (typeof  RELATION_ATTRIBUTES == "undefined")
	{
		return;
	}
	
	var list = RELATION_ATTRIBUTES;

	for (var index = 0, taskIndex = 0, crIndex = 0; index < list.length; index++)
	{
		if (list[index].name == "has_associated_cv")
		{
			relationshipsForObject[0] = new Object();
			relationshipsForObject[0].name = list[index].name;
			relationshipsForObject[0].alias = list[index].alias;
		}
		else if ((list[index].name == "associated_cv") || (list[index].name == "has_associated_task"))
		{
			relationshipsForTask[taskIndex] = new Object();
			relationshipsForTask[taskIndex].name = list[index].name;
			relationshipsForTask[taskIndex].alias = list[index].alias;
			taskIndex++
			
		}
		else
		{
			relationshipsForCR[crIndex] = new Object();
			relationshipsForCR[crIndex].name = list[index].name;
			relationshipsForCR[crIndex].alias = list[index].alias;
			crIndex++
		}
	}
}

/**
 * Get the specified relation's alias
 */
function getRelationshipAlias(relation)
{
	var list = RELATION_ATTRIBUTES;

	for (var i = 0; i < list.length; i++)
	{
		if (list[i].name == relation)
		{
			return list[i].alias;
		}
	}

	return relation;
}

/**
 * On click of new link, show the relationships dojo dialog. 
 */
function plusOnClick()
{
	if (IsFormLoading())
	{
		return;
	}

	showRelationshipInfoDialog();
	loadRelationships(dojo.byId("relationships"), fetchLastReportType());

	return;
}

/**
 * On click of ok on the dojo dialog, add the newly added subreport.
 */
function okOnClick()
{
	hideRelationshipInfoDialog();
	removePlusTab();

	var ul = document.getElementById("tabsList");
	var listControl = dojo.byId("relationships");
	var relation = listControl.options[listControl.selectedIndex].value;

	var toolTip, type, typeInString;
	var subReport;

	if (relation =="has_associated_cv" || relation == "associated_task")
	{
		toolTip = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alt00084' --><!-- /WSLET -->";
		type = SUBREPORT_TYPE_TASK;
		typeInString = "TASK"; 
	}
	else if (relation == "associated_cv" || relation == "attachment")
	{
		toolTip = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alt00085' --><!-- /WSLET -->";
		type = SUBREPORT_TYPE_OBJECT;
		typeInString = "OBJECT";
	}
	else
	{
		toolTip = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alt00083' --><!-- /WSLET -->";
		type = SUBREPORT_TYPE_CR;
		typeInString = "CHANGE_REQUEST";
	}
	
	subReport = createNewSubReport(relation, type);
	var li = createTab(getRelationshipAlias(relation), toolTip, subReport);
	
	addSubReport(subReport);
	ul.appendChild(li);

	// Append plus tab only if the relationship is not of type Attachment
	if (!(relation == "attachment"))
	{
		var plusLi = createPlusTab(); 
		ul.appendChild(plusLi);
	}

	// activate the currently added subreport tab
	activateTab(li.getElementsByTagName ("a")[0]);
	displaySortingAttributes();
	window.gScopeMenu.refreshScope();
	addSubReportToMainReport(typeInString, relation, subReport.name);
}

/**
 * creation of a new sub report.
 */
function createNewSubReport(relation, type)
{ 
	var subReport = new Object();
	subReport.name = user + "_SUBREPORT" + "_" + new Date().getTime();

	return updateSubReport(subReport, relation, type);
}

function updateSubReport(subReport, relation, type, retainAttributes)
{
	subReport.relation = relation;
	subReport.recursive = false;
	subReport.def_type =  type;

	if (! retainAttributes)
	{
		switch (type)
		{
		case SUBREPORT_TYPE_CR:
			subReport.attributes = "problem_number:0:false|crstatus:1:false|problem_synopsis:2:false";
			subReport.sort_order = "problem_number:intb:A";
			break;
		case SUBREPORT_TYPE_TASK:
			subReport.attributes = "task_number:0:false|status:1:false|task_synopsis:2:true";
			subReport.sort_order = "task_number:intb:A";
			break;
		case SUBREPORT_TYPE_OBJECT:
			if (relation == "attachment")
			{
				subReport.attributes = "attachment_name:0:true";
				subReport.sort_order = "attachment_name:string:A";
			}
			else
			{
				subReport.attributes = "displayname:0:true";
				subReport.sort_order = "displayname:string:A";
			}
			break;
		}
	}

	return subReport;	
}

function addSubReport(subReport)
{
	var subReports = getReportObject().sub_reports;
	var subPos = subReports.length;
	subReports[subPos] = subReport;
}

/**
* Create a '+' tab beside the tabbed structure in the format tab
*/
function createPlusTab()
{
	var a = document.createElement("a");
	a.href = "#";
	a.appendChild(document.createTextNode("+"));
	a.id = "plus";
	a.style.backgroundColor = "#C2CCDC";
	a.style.paddingLeft = "10px";
	a.style.paddingRight = "10px";
	a.style.className = "tabbedList";
	a.title = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='addANewRelation' --><!-- /WSLET -->";
	a.onclick = plusOnClick;
	var li = document.createElement("li");
	li.appendChild(a);

	return li;
}

/**
 * Remove the new link
 */
function removePlusTab()
{
	var elem = document.getElementById('plus');
	elem.parentNode.removeChild(elem);
}

function updatesubReportToMainReport(subReportType, subReportRelation, subReport, retainAttributes)
{
	subReport.type       = subReportType;
	subReport.relation   = subReportRelation;
	subReport.relationAlias  = getRelationshipAlias(subReportRelation);
	subReport.recursive   = "false";
	subReport.style      = MAIN_REPORT.style; 
	
	if (!retainAttributes)
	{
		subReport.attributes = new Array();
	}

	subReport.header_template = new Object();

	subReport.header_template.use_title         = false;
	subReport.header_template.use_count         = false;
	subReport.header_template.use_date          = false;
	subReport.header_template.use_report_name   = false;
	subReport.header_template.use_query_name    = false;
	subReport.header_template.use_query_string  = false;

	subReport.use_footer      = false;
	subReport.footer_template = new Object();

	subReport.footer_template.use_title         = false;
	subReport.footer_template.use_count         = false;
	subReport.footer_template.use_date          = false;
	subReport.footer_template.use_report_name   = false;
	subReport.footer_template.use_query_name    = false;
	subReport.footer_template.use_query_string  = false;
	
	loadAttributesAndSortOrderToMainRpt();
}

/**
 * Add the subreport to the main report data structure
 */
function addSubReportToMainReport(subReportType, subReportRelation, subReportName)
{
	var subPos = MAIN_REPORT.sub_reports.length;
	MAIN_REPORT.sub_reports[subPos]            = new Object();
	MAIN_REPORT.sub_reports[subPos].name       = subReportName;

	updatesubReportToMainReport(subReportType, subReportRelation, MAIN_REPORT.sub_reports[subPos]);
}

/**
 * Load the attributes and sort order on to the main report data structure.
 */
function loadAttributesAndSortOrderToMainRpt(sortOrder, subReport, mainReportDataStructure)
{
	var attrs;
	var sortAttrs;
	var currentReport; 
	
	if (subReport == undefined)
	{
		attrs = parseReportAttributes(activeSubReport.attributes, false);
		sortAttrs = parseReportSortOrder(activeSubReport.sort_order);
		currentReport = fetchCurrentReportFromMainReport();
	}
	else
	{
		attrs = parseReportAttributes(subReport.attributes, false);
		sortAttrs = parseReportSortOrder(sortOrder);
		currentReport = mainReportDataStructure;
	}
	currentReport.attributes = new Array();
	
	for (var attrIndex = 0; attrIndex < attrs.length; attrIndex++)
	{
		var sortDefined = false;
		currentReport.attributes[attrIndex] = new Object();
		currentReport.attributes[attrIndex].name = attrs[attrIndex].name; //NAME
		currentReport.attributes[attrIndex].span = attrs[attrIndex].span; //SPAN
		currentReport.attributes[attrIndex].display_position = attrs[attrIndex].position; //DISPLAY_POSITION
		
		for (var sortIndex = 0; sortIndex < sortAttrs.length; sortIndex++)
		{

			if (attrs[attrIndex].name == sortAttrs[sortIndex].name)
			{
				sortDefined=true;
				currentReport.attributes[attrIndex].sorting = new Object();
				currentReport.attributes[attrIndex].sorting.sort_position = sortIndex + 1; //SORT_POSITION
				currentReport.attributes[attrIndex].sorting.sort_type = sortAttrs[sortIndex].type; //SORT_TYPE
				currentReport.attributes[attrIndex].sorting.sort_mode = sortAttrs[sortIndex].direction;
				
				continue;
			}
		}

		if (!sortDefined)
		{
			sortDefined=true;
			currentReport.attributes[attrIndex].sorting = new Object();
			currentReport.attributes[attrIndex].sorting.sort_position = -1;
			currentReport.attributes[attrIndex].sorting.sort_type = "";
			currentReport.attributes[attrIndex].sorting.sort_mode = "";
		}
	}
}

/**
 * Update sort order on to the main report data structure.
 */
function updateSortOrderToMainReport(reportObject)
{
	var subReport;
	var mainReport;

	if (getReportObject() != undefined)
	{
		reportObject = getReportObject();
	}
		
	for ( var subReportIndex = 0; subReportIndex< reportObject.sub_reports.length; subReportIndex++)
	{
		subReport = reportObject.sub_reports[subReportIndex];

		if (subReportIndex == 0)
		{
			mainReport = MAIN_REPORT;
		}
		else
		{
			mainReport = fetchCorrespondingMainReport(subReport);
		}
		
		loadAttributesAndSortOrderToMainRpt(getSortOrder(subReportIndex), subReport, mainReport);
	}
}

function fetchCorrespondingMainReport(subReport)
{
	for (var subReportIndex = 0; subReportIndex < MAIN_REPORT.sub_reports.length; subReportIndex++)
	{
		if (subReport.name == MAIN_REPORT.sub_reports[subReportIndex].name)
		{
			return MAIN_REPORT.sub_reports[subReportIndex];
		}
	}
}

/**
 * Initialize the main report data structure only in case of adhoc relational formats
 */
function initializeMainReport()
{
	if (!isRelationalRptFormat())
	{
		return;
	}

	if (getNavFrame().isCanceledRelRpt)
	{
		MAIN_REPORT = getNavFrame().relReportDataStructure;
		getNavFrame().isCanceledRelRpt = false;
	}
	else
	{
		getNavFrame().relReportDataStructure = MAIN_REPORT;		
	}

	if ((getReportObject() != null) && (getReportObject().isNew))
	{
		MAIN_REPORT.name = user + "_MAIN_REPORT_";
		MAIN_REPORT.description = getReportObject().description;
		MAIN_REPORT.style = getReportObject().style;
		loadAttributesAndSortOrderToMainRpt();
	}

	setupRelationships();
}

/**
 * Fetch the current active report's relation type.
 */
function getRelationType()
{
	var currentReport = fetchCurrentReportFromMainReport();
	var relation;

	if (currentReport != undefined)
	{
		relation = currentReport.relation;
	}
	else
	{
		relation = activeSubReport.relation;
	}

	return relation;
}

/**
 * Fetch the relation of the last sub-report
 */
function fetchLastRelation(isCanceledRelRpt)
{
	var report = MAIN_REPORT;

	if (isCanceledRelRpt)
	{
		report = getNavFrame().relReportDataStructure;
	}

	var relation = "";

	if (report.sub_reports.length > 0)
	{
		relation = report.sub_reports[report.sub_reports.length-1].relation;
	}

	return relation;
}

/**
 * Fetch the type of the last sub-report
 */
function fetchLastReportType()
{
	var type = "CHANGE_REQUEST";

	if (MAIN_REPORT.sub_reports.length > 0)
	{
		type = MAIN_REPORT.sub_reports[ MAIN_REPORT.sub_reports.length-1].type;
	}
	
	return type;
}

/**
 * Find if the current report is recursive or not. 
 */
function isRelationshipRecursive()
{
	var currentReport = fetchCorrespondingMainReport(activeSubReport);
	var isRecursive =  false;

	if ((currentReport != undefined) && (currentReport.recursive == "true"))
	{
		isRecursive = true;
	}
	else
	{
		isRecursive = activeSubReport.recursive;
	}

	return isRecursive;
}

function recursiveOnChange()
{
	var isRecursive = document.getElementById("recursiveOnSubRpt").checked;
	
	activeSubReport.recursive = isRecursive;
	fetchCurrentReportFromMainReport().recursive = ""+ isRecursive;
}

/**
 * On change of relationship value in the listbox, the respective tab shall get
 * reloaded as per the relationship selected.
 */
function relationshipOnChange()
{
	var list = document.getElementById("relationshipType");
	var prevType = activeSubReport.def_type;
	var prevRelation = activeSubReport.relation;
	activeSubReport.relation = list.options[list.selectedIndex].value;
	fetchCurrentReportFromMainReport().relation = list.options[list.selectedIndex].value;
	fetchCurrentReportFromMainReport().relationAlias = getRelationshipAlias(list.options[list.selectedIndex].value);
	reloadTab(activeSubReport.relation, prevType, prevRelation);
}

/**
 * Reload the tab based on the relation specified.
 */
function reloadTab(relation, prevType, prevRelation)
{
	var toolTip, type, typeInString;
	var subReport;

	if (relation =="has_associated_cv" || relation == "associated_task")
	{
		toolTip = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alt00084' --><!-- /WSLET -->";
		type = SUBREPORT_TYPE_TASK;
		typeInString = "TASK";
	}
	else if (relation == "associated_cv" || relation == "attachment")
	{
		toolTip = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alt00085' --><!-- /WSLET -->";
		type = SUBREPORT_TYPE_OBJECT;
		typeInString = "OBJECT";
	}
	else
	{
		toolTip = "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='alt00083' --><!-- /WSLET -->";
		type = SUBREPORT_TYPE_CR;
		typeInString = "CHANGE_REQUEST";
	}

	var retainAttributes = false; 

	if (prevType == type)
	{
		if (type == SUBREPORT_TYPE_OBJECT)
		{
			if (!(prevRelation == "attachment") && !(relation == "attachment"))
			{
				retainAttributes = true;
			}
		}
		else
		{
			retainAttributes = true;
			activeSubReport.attributes = getSelectedAttributes();
			activeSubReport.sort_order = getSelectedSortOrder();
		}
	}
		
	updateSubReport(activeSubReport, relation, type, retainAttributes);
	updatesubReportToMainReport(typeInString, relation, fetchCurrentReportFromMainReport(), retainAttributes);

	// activate the respective tab
	var subReports = getReportObject().sub_reports;
	var ul = document.getElementById("tabsList");

	for (var subReportIndex = 0; subReportIndex < subReports.length; subReportIndex++)
	{
		if (subReports[subReportIndex].name == activeSubReport.name)
		{
			var li = ul.getElementsByTagName("a")[subReportIndex];
			li.id = getRelationshipAlias(relation);
			li.innerHTML = getRelationshipAlias(relation);
			activateTab(li);
			continue;
		}
	}

	loadSourceAttributes(activeSubReport);
	displaySortingAttributes();

	// activate or de-activate the plus tab based on the last relation type 
	var plusEnabled = false;

	if (document.getElementById('plus') != undefined)
	{
		plusEnabled = true;
	}
	
	if ((fetchLastRelation() == 'attachment') && (plusEnabled))
	{
		removePlusTab();
	}
	else if ((fetchLastRelation() != 'attachment') && !(plusEnabled))
	{
		ul.appendChild(createPlusTab());
	}
}

/**
 * Fetch the current report from the Main Report data structure 
 */
function fetchCurrentReportFromMainReport()
{
	var currentReport;

	if (isMainReport(activeSubReport))
	{
		currentReport = MAIN_REPORT;
	}
	else
	{
		for (var i = 0; i < MAIN_REPORT.sub_reports.length; i++)
		{	
			if (MAIN_REPORT.sub_reports[i].name == activeSubReport.name)
			{
				currentReport = MAIN_REPORT.sub_reports[i];
			}
		}
	}

	return currentReport;
}

/**
 * Find if the specified report is Main Report/Top level report.
 */
function isMainReport(report)
{
	var reportName = report.name;

	if ((reportName.startsWith("mainColumnCR")) || (reportName.startsWith("mainBlockCR")) || (reportName == getReportObject().name))
	{
		return true;	
	}
	else
	{
	 	return false;
	}
}

String.prototype.startsWith = function(str)
{
	return (this.match("^"+str)==str)
}

/**
 * Find if the selected format is an already saved adhoc relational format.
 */
function isSavedRelRptFormat()
{
	return (getReportObject().adhocReport != undefined);
}

function setAdhocRelationType()
{
	if ((getReportObject().name == "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='columnRelationships' --><!-- /WSLET -->") || (getReportObject().name == "<!-- WSLET CODE=PTString --><!-- PARAM NAME=RESOURCE_STRING VALUE='blockRelationships' --><!-- /WSLET -->") || (isSavedRelRptFormat()))
	{
		getNavFrame().isAdhocRelationalReport = true;
		getReportObject().adhocReport = true;
	}
	else
	{
		getNavFrame().isAdhocRelationalReport = false;
	}
}

function isRelationalRptFormat()
{
	return getNavFrame().isAdhocRelationalReport;
}

function isBlockRelationalFormat()
{
	if (isRelationalRptFormat())
	{
		return (MAIN_REPORT.style == "Block");
	}
}

function showRelationListBox()
{
	document.getElementById("relationshipSection").style.display = "";
	loadRelationships(document.getElementById("relationshipType"), "ALL");
}

/**
 * Delete the currently active subreport
 */
function deleteRelationshipOnClick()
{
	var subReportName = activeSubReport.name;
	var subReports = getReportObject().sub_reports;
	var tabToActivate = 0;

	for (var i = 0; i < subReports.length; i++)
	{
		if (subReports[i].name == subReportName)
		{
			tabToActivate = i-1;
			subReports.splice(i, 1);
			continue;
		}
	}

	if (dojo.isIE)
	{
		deleteReportFromMainReportInIE(subReportName);
	}
	else
	{
		deleteReportFromMainReport(subReportName);
	}

	createTabs();

	//activate the previous tab
	var ul = document.getElementById("tabsList");
	activateTab(ul.getElementsByTagName("a")[tabToActivate]);
}

/**
* Delete the subreport from the MAIN_REPORT data structure
*/
function deleteReportFromMainReport(subReportName)
{
	for (var subReportIndex = 0; subReportIndex < MAIN_REPORT.sub_reports.length; subReportIndex++)
	{	
		if (MAIN_REPORT.sub_reports[subReportIndex].name == subReportName)
		{
			MAIN_REPORT.sub_reports.splice(subReportIndex, 1);
			break;
		}
	}
}

/**
 * Delete the subreport from the MAIN_REPORT data structure.
 * This method is specifically used by IE.
 */
function deleteReportFromMainReportInIE(subReportName)
{
	var rpts = new Array();

	for (var subReportIndex = 0,index=0; subReportIndex < MAIN_REPORT.sub_reports.length; subReportIndex++)
	{	
		if (!(MAIN_REPORT.sub_reports[subReportIndex].name == subReportName))
		{
			rpts[index++] = MAIN_REPORT.sub_reports[subReportIndex];
		}
	}

	MAIN_REPORT.sub_reports = rpts;
}

/**
 * Update spanning to the Main Report data structure
 */
function updateSpanningOnMainreport(selectedAttribute, spanningChecked)
{
	var currentReport = fetchCurrentReportFromMainReport();

	for (var i = 0; i < currentReport.attributes.length; i++)
	{
		if (currentReport.attributes[i].name == selectedAttribute)
		{
			currentReport.attributes[i].span = spanningChecked;
		}
	}
}

/**
 * Update the title to the Main Report data structure. 
 */
function updateTitleOnMainReport()
{
	getReportTitle();
}

function updatePagingOnMainReport(paging, incrSize)
{
	MAIN_REPORT.page = paging;
	MAIN_REPORT.items_per_page = incrSize;
}
