
<!-- ----------------------------------------------------------------- -->
<!-- BEGIN trendAnalysis.js -->

/*
 * IBM and/or HCL Confidential
 * (C) Copyright IBM Corp. 2020.  All Rights Reserved.
 * (C) Copyright HCL Technologies Ltd. 2021.  All Rights Reserved.
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has been
 *  deposited with the U.S. Copyright Office.
 */

// default XML values (don't need to be explicitly set by the user)
var chartTitle  = new String("");
var chartHeight = 450;
var chartWidth  = 650;
var dateFormat  = new String("");

/*
 * Return a string that intelligently joins the given subqueries with the given
 * binary operator ('and' or 'or'). This function works correctly in the event
 * either subquery is empty.
 */
function queryConcat(subquery1, operator, subquery2)
{
	if(subquery1.length > 0)
	{
		return(subquery2.length > 0 ? (subquery1 + " " + operator + " " + subquery2) : subquery1);
	}
	else
	{
		return(subquery2.length > 0 ? subquery2 : "");
	}
}

/*
 * Select an item from a listbox based on its value.
 */
function setListBoxSelection(listbox, value)
{	
	for (var i = 0; i < listbox.length; i++)
	{
		if (listbox.options[i].value == value)
		{
			listbox.options[i].selected = true;
			break;
		}	
	}
}

/*
 * Select an item from a listbox based on its value.
 */
function setMutliSelect(listbox, values)
{
	var length = listbox.length;
	var valuesSize = values.length;
	
	for (var i = 0; i < length; i++)
	{
		for (var x = 0; x < valuesSize; x++)
		{
			if (listbox.options[i].value == values[x])
			{
				listbox.options[i].selected = true;
				break;
			}	
		}
	}
}

/*
 * Select listbox entries with the given name.
 */
function selectEntryMutli(listbox, values)
{
	var length = listbox.length;
	var valuesSize = values.length;
	
	for (var i = 0; i < length; i++)
	{
		listbox.options[i].selected = false;
	
		for (var x = 0; x < valuesSize; x++)
		{
			if (listbox.options[i].text == values[x])
			{
				listbox.options[i].selected = true;
				break;
			}	
		}
	}
}

/*
 * Select a listbox entry with the given name.
 */
function selectEntry(listbox, entryName)
{
	for (var i = 0; i < listbox.length; i++)
	{
		if (listbox.options[i].text == entryName)
		{
			listbox.options[i].selected = true;
			break;
		}
	}
}


/*
 * If the selected item for the given listbox is blank, deselect it
 * and select the first (default) entry.
 */
function deselectBlankEntry(listbox)
{
	// prevent selection of blank entries
	if (!IsFormLoading() && listbox.options[listbox.selectedIndex].text == "")
	{
		listbox.options[0].selected = true;
	}
}

/*
 * Invoke the custom query filter in a pop-up window.
 */
function filter_OnClick()
{
	window.queryFilterField = document.report.queryFilter;

	var url = "<!-- WSLET CODE=PTAdmin --><!-- PARAM NAME=SERVLET_PATH VALUE=PTWEB_SERVLET --><!-- /WSLET -->" +
	          "?ACTION_FLAG=query_form" +
	          "&TEMPLATE_FLAG=QueryFilter" +
	          "&LIFECYCLE_FLAG=" + escape(Lifecycle) +
			  "&role=" + escape(role) + 
			  "&token=" + token;

	document.open(url, "queryFilter", "resizable,width=400,height=520").focus();
}

/*
 * Load the date attributes from QUERIABLE_ATTRIBUTES_DATA into the specified listbox.
 */
function loadDateAttrs(listbox)
{
	listbox.options.length = 0;	// clear the listbox
	
	// load the date attributes into the listbox
	for (var i = 0, j = 0; i < QUERIABLE_ATTRIBUTES_DATA.length; i++)
	{
		if (QUERIABLE_ATTRIBUTES_DATA[i].type == "CCM_DATE")
		{
			listbox.options[j++] = new Option(QUERIABLE_ATTRIBUTES_DATA[i].label, QUERIABLE_ATTRIBUTES_DATA[i].name);
		}
	}
}

<!-- END trendAnalysis.js -->
<!-- ----------------------------------------------------------------- -->
